#!/usr/bin/env python
"""
Tests for ConfigSectionTree
"""

from builtins import str

import unittest

from WMCore.WMSpec.ConfigSectionTree import ConfigSectionTree
from WMCore.WMSpec.ConfigSectionTree import TreeHelper
from WMCore.WMSpec.ConfigSectionTree import findTopNode, addNode


class ConfigSectionTreeTest(unittest.TestCase):
    """
    TestCase for ConfigSectionTree

    """
    def testA(self):
        """instantiation"""
        try:
            cst = ConfigSectionTree("node1")
        except Exception as ex:
            msg = "Error instantiating ConfigSectionTree:\n"
            msg += str(ex)
            self.fail(msg)

        try:
            helper = TreeHelper(cst)
        except Exception as ex:
            msg = "Error instantiating TreeHelper:\n"
            msg += str(ex)
            self.fail(msg)

    def testB(self):
        """building tree"""

        nodeNameOrder = []
        cst1 = TreeHelper(ConfigSectionTree("node1"))
        nodeNameOrder.append(cst1.name())

        cst2a = TreeHelper(ConfigSectionTree("node2a"))
        cst2b = TreeHelper(ConfigSectionTree("node2b"))
        cst2c = TreeHelper(ConfigSectionTree("node2c"))


        cst1.addNode(cst2a)
        nodeNameOrder.append(cst2a.name())
        cst1.addNode(cst2b)
        nodeNameOrder.append(cst2b.name())
        cst1.addNode(cst2c)
        nodeNameOrder.append(cst2c.name())


        self.assertEqual(
            cst1.listNodes(), nodeNameOrder)


        cst3a = TreeHelper(ConfigSectionTree("node3a"))
        cst3b = TreeHelper(ConfigSectionTree("node3b"))
        cst3c = TreeHelper(ConfigSectionTree("node3c"))
        cst3d = TreeHelper(ConfigSectionTree("node3d"))
        cst3e = TreeHelper(ConfigSectionTree("node3e"))
        cst3f = TreeHelper(ConfigSectionTree("node3f"))
        cst3g = TreeHelper(ConfigSectionTree("node3g"))
        cst3h = TreeHelper(ConfigSectionTree("node3h"))
        cst3i = TreeHelper(ConfigSectionTree("node3i"))

        cst2a.addNode(cst3a)
        nodeNameOrder.insert(nodeNameOrder.index(cst2a.name())+1,
                             cst3a.name())
        cst2a.addNode(cst3b)
        nodeNameOrder.insert(nodeNameOrder.index(cst3a.name())+1,
                             cst3b.name())
        cst2a.addNode(cst3c)
        nodeNameOrder.insert(nodeNameOrder.index(cst3b.name())+1,
                             cst3c.name())

        cst2b.addNode(cst3d)
        nodeNameOrder.insert(nodeNameOrder.index(cst2b.name())+1,
                             cst3d.name())
        cst2b.addNode(cst3e)
        nodeNameOrder.insert(nodeNameOrder.index(cst3d.name())+1,
                             cst3e.name())
        cst2b.addNode(cst3f)
        nodeNameOrder.insert(nodeNameOrder.index(cst3e.name())+1,
                             cst3f.name())

        cst2c.addNode(cst3g)
        nodeNameOrder.insert(nodeNameOrder.index(cst2c.name())+1,
                             cst3g.name())
        cst2c.addNode(cst3h)
        nodeNameOrder.insert(nodeNameOrder.index(cst3g.name())+1,
                             cst3h.name())
        cst2c.addNode(cst3i)
        nodeNameOrder.insert(nodeNameOrder.index(cst3h.name())+1,
                             cst3i.name())

        self.assertEqual(cst1.listNodes(), nodeNameOrder)

    def testC(self):
        """parent hunting"""




        node1 = ConfigSectionTree("node1")
        node2 = ConfigSectionTree("node2")
        node3 = ConfigSectionTree("node3")


        addNode(node1, node2)
        addNode(node2, node3)


        topNode = TreeHelper(findTopNode(node3))
        self.assertEqual(topNode.name(), "node1")

        topNode = TreeHelper(findTopNode(node2))
        self.assertEqual(topNode.name(), "node1")

        topNode = TreeHelper(findTopNode(node1))
        self.assertEqual(topNode.name(), "node1")

        helper = TreeHelper(node2)
        helper.setTopOfTree()

        # searching for the tree top above the set tree top
        # means you cant even find the tree.
        # not sure if this is a problem....
        topNode = TreeHelper(findTopNode(node1))
        self.assertEqual(topNode.name(), "node1")

        topNode = TreeHelper(findTopNode(node2))
        self.assertEqual(topNode.name(), "node2")
        topNode = TreeHelper(findTopNode(node3))
        self.assertEqual(topNode.name(), "node2")









if __name__ == '__main__':
    unittest.main()



"""
Module defines REST API methods and their handles.
Implementation of handles is in corresponding modules, not here.

"""
from __future__ import print_function, division

from builtins import object
import cherrypy

from WMCore.Configuration import Configuration
from WMCore.REST.Server import RESTApi
from WMCore.REST.Services import ProcessMatrix

from WMCore.ReqMgr.ReqMgrCouch import ReqMgrCouch
from WMCore.ReqMgr.Service.Auxiliary import (Info, ReqMgrConfigData, CMSSWVersions, WMAgentConfig,
                                             CampaignConfig, UnifiedConfig, TransferInfo, ParentLocks)
from WMCore.ReqMgr.Service.RequestAdditionalInfo import (RequestSpec, WorkloadConfig, WorkloadSplitting)
from WMCore.ReqMgr.Service.Request import Request, RequestStatus, RequestType
from WMCore.ReqMgr.Service.WMStatsInfo import WMStatsInfo


class IndividualCouchManager(object):
    """
    Wrapper for the database API object, such that it's *not* shared
    among all the Rest APIs.
    """

    def __init__(self, config):
        self.db_handler = ReqMgrCouch(config)


class RestApiHub(RESTApi):
    """
    Server object for REST data access API.
    """

    def __init__(self, app, config, mount):
        """
        :arg app: reference to application object; passed to all entities.
        :arg config: reference to configuration; passed to all entities.
        :arg str mount: API URL mount point; passed to all entities."""

        RESTApi.__init__(self, app, config, mount)

        cherrypy.log("ReqMgr entire configuration:\n%s" % Configuration.getInstance())
        cherrypy.log("ReqMgr REST hub configuration subset:\n%s" % config)

        # Makes raw format as default
        # self.formats.insert(0, ('application/raw', RawFormat()))
        self._add({"about": Info(app, IndividualCouchManager(config), config, mount),
                   "info": Info(app, IndividualCouchManager(config), config, mount),
                   "app_config": ReqMgrConfigData(app, IndividualCouchManager(config), config, mount),
                   "request": Request(app, IndividualCouchManager(config), config, mount),
                   "cmsswversions": CMSSWVersions(app, IndividualCouchManager(config), config, mount),
                   "wmagentconfig": WMAgentConfig(app, IndividualCouchManager(config), config, mount),
                   "campaignconfig": CampaignConfig(app, IndividualCouchManager(config), config, mount),
                   "unifiedconfig": UnifiedConfig(app, IndividualCouchManager(config), config, mount),
                   "transferinfo": TransferInfo(app, IndividualCouchManager(config), config, mount),
                   "status": RequestStatus(app, IndividualCouchManager(config), config, mount),
                   "type": RequestType(app, IndividualCouchManager(config), config, mount),
                   "spec_template": RequestSpec(app, IndividualCouchManager(config), config, mount),
                   "workload_config": WorkloadConfig(app, IndividualCouchManager(config), config, mount),
                   "splitting": WorkloadSplitting(app, IndividualCouchManager(config), config, mount),
                   "wmstats_info": WMStatsInfo(app, IndividualCouchManager(config), config, mount),
                   "proc_status": ProcessMatrix(app, self, config, mount),
                   "parentlocks": ParentLocks(app, IndividualCouchManager(config), config, mount)
                   })



#!/usr/bin/env python
"""
_Destroy_

Tear down the MySQL version of the TestDB schema.
"""




import threading

from WMCore.Database.DBCreator import DBCreator

class Destroy(DBCreator):
    def __init__(self, logger = None, dbi = None, params = None):
        """
        __init__

        """
        myThread = threading.currentThread()
        if logger == None:
            logger = myThread.logger
        if dbi == None:
            dbi = myThread.dbi

        DBCreator.__init__(self, logger, dbi)

        self.delete["01test_tablea"] = "DROP TABLE test_tablea"
        self.delete["02test_tableb"] = "DROP TABLE test_tableb"
        self.delete["03test_tablec"] = "DROP TABLE test_tablec"
        self.delete["04test_bigcol"] = "DROP TABLE test_bigcol"

        return



from WMCore.WebTools.NestedModel import NestedModel

class DummyNestedModel(NestedModel):
    def __init__(self, config):
        '''
        Initialise the RESTModel and add some methods to it.
        '''
        NestedModel.__init__(self, config)
        self.methods = {'GET':{
                           'foo': {
                                    'default':{'default_data':1234,
                                               'call':self.foo,
                                               'version': 1,
                                               'args': ['message'],
                                               'expires': 3600,
                                               'validation': []},
                                    'ping':{'default_data':1234,
                                           'call':self.ping,
                                           'version': 1,
                                           'args': [],
                                           'expires': 3600,
                                           'validation': []}}
                           }
                    }

    def foo(self, message = None):
        """
        Return a different simple message
        """
        if message:
            return 'foo ' + message
        else:
            return 'foo'

    def ping(self):
        """
        Return a simple message
        """
        return 'ping'



#!/usr/bin/env python
"""
    WorkQueueElement unit tests
"""

from builtins import range
import unittest
import itertools

from Utils.PythonVersion import PY3
from WMCore.WorkQueue.DataStructs.WorkQueueElement import WorkQueueElement, possibleSites

RequestNames = ['test', 'test2', '', 'abcdefg']
TaskNames = [None, '', 'task1', 'task2', 'abcdfg']
Inputs = [{}, {'/dataset/RAW': ['sitea']},
          {'/dataset/RAW': ['sitea'], '/dataset/RECO': ['sitea', 'siteb']}]
Masks = [None, {'FirstEvent': 1}, {'FirstEvent': 1, 'LastEvent': 10},
         {'FirstEvent': 5, 'LastLumi': 10}]
Dbses = [None, 'https://example.com/dbs', 'http://example.com/another_dbs']
Progress = Priority = [x for x in range(0, 100, 10)]
Teams = [None, '', 'bob', 'A-Team']
ParentQueueUrl = [None, '', 'https://something']
ParentQueueId = [None, '1', 2]
Acdcs = [{}, {'Something': 'Somewhat'}]


class WorkQueueElementTest(unittest.TestCase):
    def setUp(self):
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def testId(self):
        """Id calculated correctly"""
        ele = WorkQueueElement(RequestName='test')
        self.assertEqual(ele.id, '8b2e394154f6464f4b2aaf086a45f8c2')

    def testIdUnique(self):
        """Modifying a relevant parameter varies the id"""
        ids = {}
        # Vary parameters that affect the work or input data,
        # verify each id is unique
        for params in itertools.product(RequestNames, TaskNames, Inputs,
                                        Masks, Dbses, Acdcs):
            ele = WorkQueueElement(RequestName=params[0], TaskName=params[1],
                                   Inputs=params[2], Mask=params[3],
                                   Dbs=params[4], ACDC=params[5]
                                  )
            self.assertFalse(ele.id in ids)
            ids[ele.id] = None

    def testIdIgnoresIrrelevant(self):
        """Id calculation ignores irrelavant variables"""
        ele = WorkQueueElement(RequestName='testIdIgnoresIrrelevant')
        this_id = ele.id
        for params2 in itertools.product(Progress, Priority, Teams,
                                         ParentQueueUrl, ParentQueueId):
            ele = WorkQueueElement(RequestName='testIdIgnoresIrrelevant',
                                   PercentSuccess=params2[0],
                                   Priority=params2[1], TeamName=params2[2],
                                   ParentQueueUrl=params2[3], ParentQueueId=params2[4],
                                  )
            # id not changed by changing irrelvant parameters
            self.assertEqual(ele.id, this_id)

    def testIdImmutable(self):
        """Id fixed once calculated"""
        ele = WorkQueueElement(RequestName='testIdImmutable')
        before_id = ele.id
        ele['RequestName'] = 'somethingElse'
        self.assertEqual(before_id, ele.id)

    def testSetId(self):
        """Can override id generation"""
        ele = WorkQueueElement(RequestName='testIdImmutable')
        before_id = ele.id
        ele.id = 'something_new'
        self.assertEqual('something_new', ele.id)
        self.assertNotEqual(before_id, ele.id)

    def testPassesSiteRestriction(self):
        """
        Workqueue element site restriction check (same as workRestrictions)
        """
        # test element ala MonteCarlo
        ele = WorkQueueElement(SiteWhitelist=["T1_IT_CNAF", "T2_DE_DESY"], SiteBlacklist=["T1_US_FNAL"])
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T2_CH_CERN"))
        self.assertTrue(ele.passesSiteRestriction("T1_IT_CNAF"))

        # test element with input dataset
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": []}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T2_CH_CERN"))
        self.assertFalse(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertFalse(ele.passesSiteRestriction("T2_DE_DESY"))
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_DE_DESY"]}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertTrue(ele.passesSiteRestriction("T2_DE_DESY"))

        # test element with input and parent dataset
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": []}
        ele['ParentFlag'] = True
        ele['ParentData'] = {"/MY/BLOCK2/NAME#002590494c06": []}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T2_CH_CERN"))
        self.assertFalse(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertFalse(ele.passesSiteRestriction("T2_DE_DESY"))
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_DE_DESY"]}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T2_DE_DESY"))
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_DE_DESY"]}
        ele['ParentData'] = {"/MY/BLOCK2/NAME#002590494c06": ["T1_IT_CNAF", "T2_CH_CERN", "T2_DE_DESY"]}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertTrue(ele.passesSiteRestriction("T2_DE_DESY"))

        # test element with input, parent and pileup dataset
        ele['PileupData'] = {"/MY/DATASET/NAME": []}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T2_CH_CERN"))
        self.assertFalse(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertFalse(ele.passesSiteRestriction("T2_DE_DESY"))
        ele['PileupData'] = {"/MY/DATASET/NAME": ["T2_US_Nebraska", "T1_IT_CNAF"]}
        self.assertFalse(ele.passesSiteRestriction("T1_IT_CNAF"))
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T1_IT_CNAF", "T2_DE_DESY"]}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertTrue(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertFalse(ele.passesSiteRestriction("T2_DE_DESY"))

    def testPassesSiteRestrictionLocationFlags(self):
        """
        Workqueue element site restriction check (same as workRestrictions)
        """
        # test element ala MonteCarlo
        ele = WorkQueueElement(SiteWhitelist=["T1_IT_CNAF", "T2_DE_DESY"], SiteBlacklist=["T1_US_FNAL"])
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T2_CH_CERN"))
        self.assertTrue(ele.passesSiteRestriction("T1_IT_CNAF"))

        # test element with input dataset
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": []}
        ele['NoInputUpdate'] = True
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T2_CH_CERN"))
        self.assertTrue(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertTrue(ele.passesSiteRestriction("T2_DE_DESY"))
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_DE_DESY"]}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertTrue(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertTrue(ele.passesSiteRestriction("T2_DE_DESY"))

        # test element with input and parent dataset
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": []}
        ele['ParentFlag'] = True
        ele['ParentData'] = {"/MY/BLOCK2/NAME#002590494c06": []}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T2_CH_CERN"))
        self.assertTrue(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertTrue(ele.passesSiteRestriction("T2_DE_DESY"))
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_DE_DESY"]}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertTrue(ele.passesSiteRestriction("T2_DE_DESY"))
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_DE_DESY"]}
        ele['ParentData'] = {"/MY/BLOCK2/NAME#002590494c06": ["T1_IT_CNAF", "T2_CH_CERN", "T2_DE_DESY"]}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertTrue(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertTrue(ele.passesSiteRestriction("T2_DE_DESY"))

        # test element with input, parent and pileup dataset
        ele['PileupData'] = {"/MY/DATASET/NAME": []}
        ele['NoPileupUpdate'] = True
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T2_CH_CERN"))
        self.assertTrue(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertTrue(ele.passesSiteRestriction("T2_DE_DESY"))
        ele['PileupData'] = {"/MY/DATASET/NAME": ["T2_US_Nebraska", "T1_IT_CNAF"]}
        self.assertFalse(ele.passesSiteRestriction("T2_US_Nebraska"))
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T1_IT_CNAF", "T2_DE_DESY"]}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertTrue(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertTrue(ele.passesSiteRestriction("T2_DE_DESY"))
        # only the pileup flag enabled now
        ele['NoInputUpdate'] = False
        ele['PileupData'] = {"/MY/DATASET/NAME": []}
        self.assertFalse(ele.passesSiteRestriction("T1_US_FNAL"))
        self.assertFalse(ele.passesSiteRestriction("T2_CH_CERN"))
        self.assertTrue(ele.passesSiteRestriction("T1_IT_CNAF"))
        self.assertTrue(ele.passesSiteRestriction("T2_DE_DESY"))

    def testPossibleSites(self):
        """
        Workqueue element data location check (same as workRestrictions)
        """
        # test element ala MonteCarlo
        ele = WorkQueueElement(SiteWhitelist=["T1_IT_CNAF", "T2_DE_DESY"])
        self.assertItemsEqual(possibleSites(ele), ["T1_IT_CNAF", "T2_DE_DESY"])
        # test element with InputDataset but no location
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": []}
        self.assertEqual(possibleSites(ele), [])
        # test element with InputDataset and no match location
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_CH_CERN"]}
        self.assertEqual(possibleSites(ele), [])
        # test element with InputDataset and valid location
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_CH_CERN", "T2_DE_DESY"]}
        self.assertEqual(possibleSites(ele), ["T2_DE_DESY"])

        # test element with InputDataset and ParentData with no location
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_CH_CERN", "T2_DE_DESY"]}
        ele['ParentFlag'] = True
        ele['ParentData'] = {"/MY/BLOCK2/NAME#002590494c06": []}
        self.assertEqual(possibleSites(ele), [])
        # test element with InputDataset and ParentData with no match location
        ele['ParentData'] = {"/MY/BLOCK2/NAME#002590494c06": ["T1_IT_CNAF"]}
        self.assertEqual(possibleSites(ele), [])
        # test element with InputDataset and ParentData with valid location
        ele['ParentData'] = {"/MY/BLOCK2/NAME#002590494c06": ["T1_US_FNAL", "T2_DE_DESY"]}
        self.assertEqual(possibleSites(ele), ["T2_DE_DESY"])

        # test element with InputDataset, PileupData and ParentData with no location
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_CH_CERN", "T2_DE_DESY"]}
        ele['ParentData'] = {"/MY/BLOCK2/NAME#002590494c06": ["T2_DE_DESY"]}
        ele['PileupData'] = {"/MY/DATASET/NAME": []}
        self.assertEqual(possibleSites(ele), [])
        # test element with InputDataset, PileupData and ParentData with no match location
        ele['PileupData'] = {"/MY/DATASET/NAME": ["T1_IT_CNAF", "T2_CH_CERN"]}
        self.assertEqual(possibleSites(ele), [])
        # test element with InputDataset, PileupData and ParentData with valid location
        ele['PileupData'] = {"/MY/DATASET/NAME": ["T1_IT_CNAF", "T2_DE_DESY"]}
        self.assertEqual(possibleSites(ele), ["T2_DE_DESY"])

    def testPossibleSitesLocationFlags(self):
        """
        Workqueue element data location check, using the input and PU data location flags
        """
        ele = WorkQueueElement(SiteWhitelist=["T1_IT_CNAF", "T2_DE_DESY"])

        # test element with InputDataset and no location, but input flag on
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": []}
        ele['NoInputUpdate'] = True
        self.assertItemsEqual(possibleSites(ele), ["T1_IT_CNAF", "T2_DE_DESY"])
        # test element with InputDataset and one match, but input flag on
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_IT_CNAF", "T2_CH_CERN"]}
        self.assertItemsEqual(possibleSites(ele), ["T1_IT_CNAF", "T2_DE_DESY"])
        # test element with InputDataset and one match, but pu flag on
        ele['NoInputUpdate'] = False
        ele['NoPileupUpdate'] = True
        self.assertEqual(possibleSites(ele), ["T1_IT_CNAF"])
        # test element with InputDataset and one match, but both flags on
        ele['NoInputUpdate'] = True
        self.assertItemsEqual(possibleSites(ele), ["T1_IT_CNAF", "T2_DE_DESY"])

        # test element with InputDataset and ParentData and no location, but both flags on
        ele['ParentFlag'] = True
        ele['ParentData'] = {"/MY/BLOCK2/NAME#002590494c06": []}
        self.assertItemsEqual(possibleSites(ele), ["T1_IT_CNAF", "T2_DE_DESY"])
        # test element with InputDataset and ParentData and no location, but input flag on
        ele['NoPileupUpdate'] = False
        self.assertItemsEqual(possibleSites(ele), ["T1_IT_CNAF", "T2_DE_DESY"])
        # test element with InputDataset and ParentData and no location, but pileup flag on
        ele['NoInputUpdate'] = False
        ele['NoPileupUpdate'] = True
        self.assertEqual(possibleSites(ele), [])

        # test element with InputDataset, PileupData and ParentData with no location, but pileup flag on
        ele['Inputs'] = {"/MY/BLOCK/NAME#73e99a52": ["T1_US_FNAL", "T2_CH_CERN", "T2_DE_DESY"]}
        ele['ParentData'] = {"/MY/BLOCK2/NAME#002590494c06": ["T2_DE_DESY"]}
        ele['PileupData'] = {"/MY/DATASET/NAME": []}
        self.assertEqual(possibleSites(ele), ["T2_DE_DESY"])
        # test element with InputDataset, PileupData and ParentData with no location, but both flags on
        ele['NoInputUpdate'] = True
        self.assertItemsEqual(possibleSites(ele), ["T1_IT_CNAF", "T2_DE_DESY"])
        # test element with InputDataset, PileupData and ParentData with no location, but input flag on
        ele['NoPileupUpdate'] = False
        self.assertEqual(possibleSites(ele), [])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python

"""
_CondorPluginProfile_

CondorPluginProfile unittests
"""
import cProfile
import os.path
import pstats
import unittest

from WMCore_t.BossAir_t.BossAir_t import BossAirTest, getCondorRunningJobs
from nose.plugins.attrib import attr

from WMComponent.JobSubmitter.JobSubmitterPoller import JobSubmitterPoller
from WMCore.BossAir.BossAirAPI import BossAirAPI
from WMCore.JobStateMachine.ChangeState import ChangeState


class CondorPluginProfileTest(BossAirTest):
    """
    _CondorPluginProfileTest_

    Inherit everything from BossAir
    """

    @attr('integration')
    def testF_WMSMode(self):
        """
        _WMSMode_

        Try running things in WMS Mode.
        """

        nRunning = getCondorRunningJobs(self.user)
        self.assertEqual(nRunning, 0, "User currently has %i running jobs.  Test will not continue" % (nRunning))

        config = self.getConfig()
        config.BossAir.pluginName = 'PyCondorPlugin'
        config.BossAir.submitWMSMode = True

        baAPI = BossAirAPI(config=config, insertStates=True)

        workload = self.createTestWorkload()

        workloadName = "basicWorkload"

        changeState = ChangeState(config)

        nSubs = 50
        nJobs = 100

        dummyCacheDir = os.path.join(self.testDir, 'CacheDir')

        jobGroupList = self.createJobGroups(nSubs=nSubs, nJobs=nJobs,
                                            task=workload.getTask("ReReco"),
                                            workloadSpec=os.path.join(self.testDir,
                                                                      'workloadTest',
                                                                      workloadName),
                                            site=None)
        for group in jobGroupList:
            changeState.propagate(group.jobs, 'created', 'new')

        jobSubmitter = JobSubmitterPoller(config=config)

        jobSubmitter.algorithm()

        nRunning = getCondorRunningJobs(self.user)
        self.assertEqual(nRunning, nSubs * nJobs)

        baAPI.track()
        idleJobs = baAPI._loadByStatus(status='Idle')

        baAPI.kill(jobs=idleJobs)

        del jobSubmitter

        return

    @attr('integration')
    def testT_updateJobInfo(self):
        """
        _updateJobInfo_

        Test the updateSiteInformation method from PyCondorPlugin.py
        """

        nRunning = getCondorRunningJobs(self.user)
        self.assertEqual(nRunning, 0, "User currently has %i running jobs.  Test will not continue" % (nRunning))

        config = self.getConfig()
        config.BossAir.pluginName = 'PyCondorPlugin'
        config.BossAir.submitWMSMode = True

        baAPI = BossAirAPI(config=config, insertStates=True)
        workload = self.createTestWorkload()
        workloadName = "basicWorkload"
        changeState = ChangeState(config)

        nSubs = 1
        nJobs = 2
        dummycacheDir = os.path.join(self.testDir, 'CacheDir')
        jobGroupList = self.createJobGroups(nSubs=nSubs, nJobs=nJobs,
                                            task=workload.getTask("ReReco"),
                                            workloadSpec=os.path.join(self.testDir,
                                                                      'workloadTest',
                                                                      workloadName),
                                            site="se.T2_US_UCSD")
        for group in jobGroupList:
            changeState.propagate(group.jobs, 'created', 'new')
        jobSubmitter = JobSubmitterPoller(config=config)
        jobSubmitter.algorithm()
        nRunning = getCondorRunningJobs(self.user)
        self.assertEqual(nRunning, nSubs * nJobs)

        baAPI.track()
        idleJobs = baAPI._loadByStatus(status='Idle')

        ##
        # Make one of the sites in the sitelist to be True for ABORTED/DRAINING/DOWN
        # updateSiteInformation() method should edit the classAd for all the jobs
        # that are bound for the site
        # Check the Q manually using condor_q -l <job id>
        jtok = baAPI.updateSiteInformation(idleJobs, "T2_US_UCSD", True)
        if jtok != None:
            baAPI.kill(jtok, errorCode=71301)  # errorCode can be either 71301/71302/71303 (Aborted/Draining/Down)

        return

    def createProfile(self, name, function):
        fileName = name
        prof = cProfile.Profile()
        prof.runcall(function)
        prof.dump_stats(fileName)
        p = pstats.Stats(fileName)
        p.strip_dirs().sort_stats('cumulative').print_stats(0.1)
        p.strip_dirs().sort_stats('time').print_stats(0.1)
        p.strip_dirs().sort_stats('calls').print_stats(0.1)
        # p.strip_dirs().sort_stats('name').print_stats(10)

    def ProfileWMSMode(self):
        self.createProfile('PyCondorProfile.prof', self.testF_WMSMode)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
"""
_Lexicon_t_

General test of Lexicon

"""

import unittest
import os
import json

from WMCore.WMBase import getTestBase
from WMCore.Lexicon import *
from WMCore.Lexicon import _gpuInternalParameters


class LexiconTest(unittest.TestCase):
    def testDBSUser(self):

        u1 = '/DC=org/DC=doegrids/OU=People/CN=Ajit Kumar Mohapatra 867118'
        u2 = '/C=IT/O=INFN/OU=Personal Certificate/L=Bari/CN=antonio pierro'
        u3 = '/DC=ch/DC=cern/OU=computers/CN=vocms39.cern.ch'
        u4 = '/C=BE/O=BEGRID/OU=IIHE(VUB)/OU=DNTK/CN=Maes Joris'
        u5 = 'cmsprod@vocms19.cern.ch'
        u6 = 'sfoulkes'
        u7 = '/DC=org/DC=doegrids/OU=People/CN=Si Xie 523253'
        u8 = '/C=IT/O=INFN/OU=Personal Certificate/L=Sns/CN=Federico Calzolari'
        u9 = '/O=GermanGrid/OU=RWTH/CN=Maarten Thomas'
        u10 = 'cmsdataops@cmssrv44.fnal.gov'
        u11 = '/DC=ch/DC=cern/OU=Organic Units/OU=Users/CN=ceballos/CN=488892/CN=Guillelmo Gomez Ceballos'
        for test_u in [u1, u2, u3, u4, u5, u6, u7, u8, u9, u10, u11]:
            assert DBSUser(test_u), 'valid search not validated'

        u10 = 'Fred Bloggs'
        u11 = 'cmsprod@vocms19#'
        u12 = 'cms-xen@drop/'
        u13 = 'C=IT/O=INFN/OU=Personal Certificate/L=Sns/CN=Federico Calzolari'
        for test_u in [u10, u11]:
            self.assertRaises(AssertionError, DBSUser, test_u)

    def testSearchDataset(self):
        ds1 = '/Higgs/blah-v2/RECO'
        ds2 = '/Higgs*/blah-v2/RECO'
        ds3 = '/*/blah-v2/RECO'
        ds4 = '/Higgs/blah*/RECO'
        ds5 = '/Higgs/blah-v2/*'
        ds6 = '/*/*/RECO'
        ds7 = '/Higgs/*/*'
        ds8 = '/*/blah-v2/*'
        ds9 = '/QCD_EMenriched_Pt30to80/Summer08_IDEAL_V11_redigi_v2/GEN-SIM-RAW'
        ds10 = '/*'
        ds11 = '/*QCD'
        ds12 = '/QCD*/Summer*'
        ds13 = '/QCD_EMenriched_Pt30to80/Summer08_IDEAL_V11_redigi_v2/GEN-SIM-*'
        ds14 = '/QCD_EMenriched_Pt30to80/Summer08_IDEAL_V11_redigi_v2/*-SIM-*'

        for test_ds in [ds1, ds2, ds3, ds4, ds5, ds6, ds9, ds10, ds11, ds12, ds13, ds14]:
            assert searchdataset(test_ds), 'valid search not validated'

        ds1 = '*/blah-v2/RECO'
        ds2 = '*/blah-v2/*'
        ds3 = '*/*//RECO'
        ds4 = '/*/*/*/*'
        ds5 = '*'
        ds6 = '/Higgs/ /RECO'
        ds7 = '/Higgs/%/RECO'
        ds8 = 'Higgs'
        ds9 = 'RECO'
        ds10 = 'blah-v2'
        ds11 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/RECO#bdd066ce-e8fb-488e-beb1-20432d96baaa'
        for test_ds in [ds1, ds2, ds3, ds4, ds5, ds6, ds7, ds8, ds9, ds10, ds11]:
            self.assertRaises(AssertionError, searchdataset, test_ds)

    def testSearchBlock(self):
        ds1 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/GEN-SIM-RAW#bdd066ce-e8fb-488e-beb1-20432d96baaa'
        ds2 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/RECO#*'
        ds3 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/*#bdd066ce-e8fb-488e-beb1-20432d96baaa'
        ds4 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/RECO#bdd066ce-e8fb-488e-beb1-*'
        ds5 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/RECO#bdd066ce-e8fb-488e-beb1-20432d96baaa'
        ds6 = '/*Minimum*'
        ds7 = '/*'
        ds8 = '/*/BeamCommissioning09*/*#bdd066ce-e8fb-488e-beb1-2043'
        ds9 = '/*/*/RECO#bdd066ce-e8fb-488e-beb1-20432d96baaa'
        ds10 = '/*/RECO#bdd066ce-e8fb-488e-beb1-20432d96baaa'
        ds11 = '/*RECO*/*#bdd066ce-e8fb-488e-beb1-20432d96baaa'
        ds12 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/RECO*'
        ds13 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/RECO*#*1234-5677-abcdessssssssss-123444'

        for test_ds in [ds1, ds2, ds3, ds4, ds5, ds6, ds7, ds8, ds9, ds10, ds11, ds12, ds13]:
            assert searchblock(test_ds), 'valid search not validated'

        ds1 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/RECO#bdd066ce-e8fb-488e/beb1/20432d96baaa'
        ds2 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/RECObdd066ce-e8fb-488e-beb1-20432d96baaa'
        ds3 = '/MinimumBias/BeamCommissioning09-PromptReco-v2/RECO/bdd066ce-e8fb-488e-beb1-20432d96baaa'
        ds4 = '/RECO#bdd066ce-e8fb-488e-beb1-20432d96baaa'
        ds5 = '/*#bdd066ce-e8fb-488e-beb1-20432d96baaa'

        for test_ds in [ds1, ds2, ds3, ds4, ds5]:
            self.assertRaises(AssertionError, searchblock, test_ds)

    def testPublishdatasetname(self):
        assert publishdatasetname('pog-Summer09-MC_31X_V3_SD_ZeroBias-v1_Full_v0CandProducerPAT'), \
            'valid publishdatasetname not validated'
        # shouldn't contain .
        self.assertRaises(AssertionError, publishdatasetname, 'withdot.pre3_IDEAL_30X_v1')

    def testGoodProcdataset(self):
        # Check that valid procdataset work
        assert procdataset('CMSSW_4_4_0_pre7_g494p02-START44_V2_special_110808-v1'), 'valid procdataset not validated'
        assert procdataset('Summer11-PU_S4_START42_V11-v1'), 'valid procdataset not validated'
        assert procdataset('CMSSW_3_0_0_pre3_IDEAL_30X-v1'), 'valid procdataset not validated'
        assert procdataset('CMSSW_3_0_0_pre3_IDEAL_30X-my_filter-my_string-v1'), 'valid procdataset not validated'
        longStr = 'a' * 99 + "-" + 'b' * 96 + "-v1"
        assert procdataset(longStr), 'valid procdataset %s length fail' % len(longStr)

    def testBadProcdataset(self):
        # Check that invalid Procdataset raise an exception
        self.assertRaises(AssertionError, procdataset, 'Drop Table')
        self.assertRaises(AssertionError, procdataset, 'Alter Table')
        self.assertRaises(AssertionError, procdataset, 'CMSSW_3_0_0_pre3_IDEAL_30X_v1')
        self.assertRaises(AssertionError, procdataset, '')
        self.assertRaises(AssertionError, procdataset, None)
        longStr = 'a' * 100 + "-" + 'b' * 96 + "-v1"
        self.assertRaises(AssertionError, procdataset, longStr)

    def testGoodUserProcDataset(self):
        dsList = ['weinberg-StealthSusy_mm16_RECO_AOD_Z2-689dc471cdaaa10be587d0cc7c95f00f',
                  'tracker-pog-Summer09-MC_31X_V3_SD_ZeroBias-v1_Full_v0CandProducerPAT-6b5c47aa1f79fc09d5b81a20702e3621',
                  'StoreResults-Summer12_DR53X-PU_S10_START53_V7A-v1_TLBSM_53x_v3_bugfix_v1-99bd99199697666ff01397dad5652e9e']
        for ds in dsList:
            self.assertTrue(userprocdataset(ds))
        longStr = 'a' * 99 + '-' + 'b' * 66 + '-' + 'c' * 32
        assert userprocdataset(longStr), 'valid userdataset %s length fail' % len(longStr)

    def testBadUserProcDataset(self):
        dsList = ['weinberg-StealthSusy_mm16_RECO_AOD_Z2-689dc471cdaaa10be587d0cc7c95z00f',
                  'weinberg-StealthSusy_mm16_RECO_AOD_Z2-689dc471cdaaa10be587d0cc7c95z00f1'
                  'tracker-pog-Summer09-MC_31X_V3_SD_ZeroBias-v1_Full_v0#CandProducerPAT-6b5c47aa1f79fc09d5b81a20702e3621']
        for ds in dsList:
            self.assertRaises(AssertionError, userprocdataset, ds)
        # 201 length
        longStr = 'a' * 100 + '-' + 'b' * 66 + '-' + 'c' * 32
        self.assertRaises(AssertionError, userprocdataset, longStr)

    def testGoodPrimdataset(self):
        # Check that valid Primdataset work
        assert primdataset('qqH125-2tau'), 'valid primdataset not validated'
        assert primdataset('RelVal124QCD_pt120_170'), 'valid primdataset not validated'
        assert primdataset('RelVal160pre14SingleMuMinusPt10'), 'valid primdataset not validated'
        longStr = 'a' * 99
        assert primdataset(longStr), 'valid primdataset %s length failed' % len(longStr)

    def testBadPrimdataset(self):
        # Check that invalid Primdataset raise an exception
        self.assertRaises(AssertionError, primdataset, 'Drop Table')
        self.assertRaises(AssertionError, primdataset, 'Alter Table')
        longStr = 'a' * 100
        self.assertRaises(AssertionError, primdataset, longStr)

    def testGoodBlock(self):

        bList = [
            "/ZPrimeToTTJets_M500GeV_W5GeV_TuneZ2star_8TeV-madgraph-tauola/StoreResults-Summer12_DR53X-PU_S10_START53_V7A-v1_TLBSM_53x_v3_bugfix_v1-99bd99199697666ff01397dad5652e9e/USER#620a38a9-29ba-4af4-b650-e2ba07d133f3",
            "/DoubleMu/aburgmei-Run2012A_22Jan2013_v1_RHembedded_trans1_tau121_ptelec1_17elec2_8_v4-f456bdbb960236e5c696adfe9b04eaae/USER#1f1eee22-cdee-0f1b-271b-77a7f559e7dd"]
        for bk in bList:
            assert block(bk), "validation failed"
        longStr = "/" + 'a' * 99 + "/" + 'a' * 199 + "/" + 'A' * 99 + "#" + 'a' * 99
        assert block(longStr), 'valid block %s length failed' % len(longStr)

    def testBadBlock(self):

        bList = ["/ZPrimeToTTJets/StoreResults-Summer12_DR53X-P",
                 "/DoubleMu/aburgme/USER1f1eee22-cdee-0f1b-271b-77a7f559e7dd"]
        for bk in bList:
            self.assertRaises(AssertionError, block, bk)
        longStr = "/" + 'a' * 100 + "/" + 'a' * 200 + "/" + 'a' * 99 + "#" + 'a' * 99
        self.assertRaises(AssertionError, block, longStr)

    def testProcVersion(self):
        """
        _testProcVersion_

        Test whether we can correctly identify
        a good processing version
        """
        self.assertRaises(AssertionError, procversion, 'version')
        self.assertRaises(AssertionError, procversion, 'a280')
        self.assertTrue(procversion('1'))
        self.assertTrue(procversion('88'))
        return

    def testGoodAcqName(self):
        """
        _testGoodAcqName_

        Test some valid AcquisitionEra names
        """
        self.assertTrue(acqname('a22'))
        self.assertTrue(acqname('aForReals'))
        self.assertTrue(acqname('Run1016B'))
        self.assertTrue(acqname('CMSSW_8_0_16_patch1'))
        return

    def testBadAcqName(self):
        """
        _testBadAcqName_

        Test some invalid AcquisitionEra names
        """
        self.assertRaises(AssertionError, acqname, '*Nothing')
        self.assertRaises(AssertionError, acqname, '1version')
        self.assertRaises(AssertionError, acqname, '_version')
        self.assertRaises(AssertionError, acqname, 'AcqEra Spaced')
        self.assertRaises(AssertionError, acqname, '')
        self.assertRaises(AssertionError, acqname, None)
        self.assertRaises(AssertionError, acqname, 'Run2016B-Terrible')
        return

    def testGoodCampaign(self):
        """
        _testGoodCampaign_

        Test some valid Campaign names
        """
        self.assertTrue(campaign(''))
        self.assertTrue(campaign(None))
        self.assertTrue(campaign('a22'))
        self.assertTrue(campaign('aForReals'))
        self.assertTrue(campaign('Run1016B'))
        self.assertTrue(campaign('CMSSW_9_1_0_pre2__UPSG_Tracker_PU200-1492810586'))
        return

    def testBadCampaign(self):
        """
        _testBadCampaign_

        Test some invalid Campaign names
        """
        self.assertRaises(AssertionError, campaign, '*Nothing')
        self.assertRaises(AssertionError, campaign, 'B@dCampaign')
        self.assertRaises(AssertionError, campaign, '\version')
        self.assertRaises(AssertionError, campaign, '/bad-campaign')
        self.assertRaises(AssertionError, campaign, 'spaced campaign')
        self.assertRaises(AssertionError, campaign, 'a_very_very_very__very__very__very__long_long_long_long_invalid_campaign_with_81_chars')
        return

    def testGoodSearchstr(self):
        # Check that valid searchstr work
        assert searchstr('/store/mc/Fall08/BBJets250to500-madgraph/*'), 'valid search string not validated'
        assert searchstr(
            '/QCD_EMenriched_Pt30to80/Summer08_IDEAL_V11_redigi_v2/GEN-SIM-RAW#cfc0a501-e845-4576-af8a-f811165d82d9'), 'valid search string not validated'
        assert searchstr('STREAMER'), 'valid search string not validated'

    def testBadSearchstr(self):
        # Check that invalid searchstr raise an exception
        self.assertRaises(AssertionError, searchstr, 'Drop Table')
        self.assertRaises(AssertionError, searchstr, 'Alter Table')

    def testGoodNamestr(self):
        # Check that valid namestr work
        assert namestr('LUMI-VDM'), 'valid name string not validated'
        assert namestr('PhysVal'), 'valid name string not validated'
        assert namestr('cosmic'), 'valid name string not validated'

    def testBadNamestr(self):
        # Check that invalid namestr raise an exception
        self.assertRaises(AssertionError, namestr, 'insert into Table')
        self.assertRaises(AssertionError, namestr, 'drop database')

    def testGoodSiteTier(self):
        # Check that valid tiers work
        assert sitetier('T0'), 'valid tier not validated'
        assert sitetier('T1'), 'valid tier not validated'
        assert sitetier('T2'), 'valid tier not validated'
        assert sitetier('T3'), 'valid tier not validated'

    def testBadSiteTier(self):
        # Check that invalid tiers raise an exception
        self.assertRaises(AssertionError, sitetier, 'T4')
        self.assertRaises(AssertionError, sitetier, 'D0')

    def testGoodJobranges(self):
        # Check that valid tiers work
        assert jobrange('3'), 'valid job range not validated'
        assert jobrange('78'), 'valid job range not validated'
        assert jobrange('3-9'), 'valid job range not validated'
        assert jobrange('32-97'), 'valid job range not validated'
        assert jobrange('2,3'), 'valid job range not validated'
        assert jobrange('24,39'), 'valid job range not validated'
        assert jobrange('2-91,45,5,64,7,78-91'), 'valid job range not validated'

    def testBadJobranges(self):
        # Check that invalid tiers raise an exception
        self.assertRaises(AssertionError, jobrange, 'I')
        self.assertRaises(AssertionError, jobrange, '1-2i')
        self.assertRaises(AssertionError, jobrange, '1-i2')
        self.assertRaises(AssertionError, jobrange, '1,2-3,3d-5')
        self.assertRaises(AssertionError, jobrange, '-1,2-3,3d-5')
        self.assertRaises(AssertionError, jobrange, '1-2,2,-5')

    def testGoodCMSName(self):
        # Check that full names work
        assert cmsname('T0_CH_CERN'), 'valid CMS name not validated'
        assert cmsname('T2_UK_SGrid_Bristol'), 'valid CMS name not validated'
        assert cmsname('T2_FR_CCIN2P3'), 'valid CMS name not validated'
        assert cmsname('T3_US_FNAL_LPC_Cloud'), 'valid CMS name not validated'

    def testBadCMSName(self):
        # Check that invalid names raise an exception
        self.assertRaises(AssertionError, cmsname, 'T5_UK_SGrid_Bristol')
        self.assertRaises(AssertionError, cmsname, 'T2-UK-SGrid_Bristol')
        self.assertRaises(AssertionError, cmsname, 'T2_UK_SGrid_Bris-tol')
        self.assertRaises(AssertionError, cmsname, 'D2_UK_SGrid_Bristol')
        self.assertRaises(AssertionError, cmsname, 'T2asjkjhadshjkdashjkasdkjhdas')
        self.assertRaises(AssertionError, cmsname, 'T2_UK')

    def testGoodIdentifier(self):
        for ok in ['__wil.1.am__', '.']:
            assert identifier(ok)

    def testBadIdentifier(self):
        for notok in ['ke$ha', '<begin>']:
            self.assertRaises(AssertionError, identifier, notok)

    def testGoodDataset(self):
        assert dataset('/a/b/C')
        assert dataset('/m000n/RIII-ver/WIDER-THAN-MILES')

    def testBadDataset(self):
        for notok in ['/Sugar/Sugar', '/Oh/honey/honey!', '/You/are/my/candy/GIIIRRL']:
            self.assertRaises(AssertionError, dataset, notok)

    def testVersion(self):
        for ok in ['CMSSW_3_8_0_pre1', 'CMSSW_1_2_0', 'CMSSW_4_0_0_patch11', 'CMSSW_3_10_0_pre9G493']:
            assert cmsswversion(ok)

    def testBadVersion(self):
        for notok in ['ORCA_3_8_0', 'CMSSW_3_5']:
            self.assertRaises(AssertionError, cmsswversion, notok)

    def testGoodCouchUrl(self):
        for ok in ['http://vittoria@antimatter.cern.ch:5984',
                   'https://fbi.fnal.gov:5984',
                   'http://fmulder:trustno1@fbi.fnal.gov:5984',
                   'http://localhost:443',
                   'http://127.0.0.1:1234',
                   'http://0.0.0.0:4321',
                   'http://1.2.3.4:5678',
                   'http://han:solo@1.2.3.4:9876',
                   'http://luke:skywalker@localhost:7654/some_db/some_doc',
                   'https://cmsreqmgr.cern.ch/couchdb/db1/doc',
                   'http://1.2.3.4:5184/somedb/some_dir/doc',
                   'http://iam.anexternal.host:5184/somedb/some_doc/some_doc']:
            assert couchurl(ok)

    def testBadCouchUrl(self):
        for notok in ['agent86@control.fnal.gov:5984', 'http:/localhost:443', 'http://www.myspace.com']:
            self.assertRaises(AssertionError, couchurl, notok)

    def testHNName(self):
        """
        _testHNName_

        Test the HN name checker
        """

        hnName('ewv2')
        hnName('m.cinquilli')
        self.assertRaises(AssertionError, hnName, 'invalid-user')

    def testLFN(self):
        """
        _testLFN_

        Test the LFN checker in several modes, including user LFNs
        """
        lfnA = '/store/mc/Fall10/DYToMuMu_M-20_TuneZ2_7TeV-pythia6/AODSIM/START38_V12-v1/0003/C0F3344F-6EC8-DF11-8ED6-E41F13181020.root'
        lfn(lfnA)
        lfnA = '/store/mc/2008/2/21/FastSim-CSA07Electron-1203615548/0009/B6E531DD-99E1-DC11-9FEC-001617E30D4A.root'
        lfn(lfnA)
        lfnA = '/store/temp/user/ewv/Higgs-123/PrivateSample/v1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/temp/user/cinquilli.nocern/Higgs-123/PrivateSample/v1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/user/ewv/Higgs-123/PrivateSample/v1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/user/cinquilli.nocern/Higgs-123/PrivateSample/v1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/temp/group/Exotica/Higgs-123/PrivateSample/v1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/group/Exotica/Higgs-123/PrivateSample/v1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/temp1/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/temp/lustre1/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/backfill/1/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/data/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/data/Run2010A/Cosmics/RECO/v4/000/143/316/0000/F65F4AFE-14AC-DF11-B3BE-00215E21F32E.root'
        lfn(lfnA)
        lfnA = '/store/data/Run2010A/Cosmics/RECO/v4/000/143/316/F65F4AFE-14AC-DF11-B3BE-00215E21F32E.root'
        lfn(lfnA)
        lfnA = '/store/hidata/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        lfn(lfnA)
        lfnA = '/store/hidata/Run2010A/Cosmics/RECO/v4/000/143/316/0000/F65F4AFE-14AC-DF11-B3BE-00215E21F32E.root'
        lfn(lfnA)
        lfnA = '/store/hidata/HIRun2011/HIMinBiasUPC/RECO/PromptReco-v1/000/182/591/449805F5-7F1B-E111-AC84-E0CB4E55365D.root'
        lfn(lfnA)
        lfnA = '/store/t0temp/data/Run2010A/Cosmics/RECO/v4/000/143/316/0000/F65F4AFE-14AC-DF11-B3BE-00215E21F32E.root'
        lfn(lfnA)
        lfnA = '/store/unmerged/data/Run2010A/Cosmics/RECO/v4/000/143/316/0000/F65F4AFE-14AC-DF11-B3BE-00215E21F32E.root'
        lfn(lfnA)
        lfnA = '/store/himc/Run2010A/Cosmics/RECO/v4/000/143/316/0000/F65F4AFE-14AC-DF11-B3BE-00215E21F32E.root'
        lfn(lfnA)
        lfnA = '/store/backfill/1/data/Run2010A/Cosmics/RECO/v4/000/143/316/0000/F65F4AFE-14AC-DF11-B3BE-00215E21F32E.root'
        lfn(lfnA)
        lfnA = '/store/backfill/1/t0temp/data/Run2010A/Cosmics/RECO/v4/000/143/316/0000/F65F4AFE-14AC-DF11-B3BE-00215E21F32E.root'
        lfn(lfnA)
        lfnA = '/store/backfill/1/unmerged/data/Run2010A/Cosmics/RECO/v4/000/143/316/0000/F65F4AFE-14AC-DF11-B3BE-00215E21F32E.root'
        lfn(lfnA)
        lfnA = '/store/backfill/1/Run2012B/Cosmics/RAW-RECO/PromptSkim-v1/000/194/912/00000/F65F4AFE-14AC-DF11-B3BE-00215E21F32E.root'
        lfn(lfnA)
        lfnA = '/store/results/qcd/QCD_Pt80/StoreResults-Summer09-MC_31X_V3_7TeV-Jet30U-JetAODSkim-0a98be42532eba1f0545cc9b086ec3c3/QCD_Pt80/USER/StoreResults-Summer09-MC_31X_V3_7TeV-Jet30U-JetAODSkim-0a98be42532eba1f0545cc9b086ec3c3/0000/C44630AC-C0C7-DE11-AD4E-0019B9CAC0F8.root'
        lfn(lfnA)
        lfnA = '/store/results/qcd/StoreResults/QCD_Pt_40_2017_14TeV_612_SLHC6_patch1/USER/QCD_Pt_40_2017_14TeV_612_SLHC6_patch1_6be6d116203e430d91d7e1d6d9a88cd7-v1/00000/028DDC2A-63A8-E311-BB40-842B2B5546DE.root'
        lfn(lfnA)
        lfnA = '/store/user/fanzago/RelValZMM/FanzagoTutGrid/f30a6bb13f516198b2814e83414acca1/outfile_10_2_tw4.root'
        lfn(lfnA)
        lfnA = '/store/group/higgs/SDMu9_Zmumu/Zmumu/OctX_HZZ3lepSkim_SDMu9/1eb161a436e69f7af28d18145e4ce909/3lepSkim_SDMu9_1.root'
        lfn(lfnA)
        lfnA = '/store/group/e-gamma_ecal/SDMu9_Zmumu/Zmumu/OctX_HZZ3lepSkim_SDMu9/1eb161a436e69f7af28d18145e4ce909/3lepSkim_SDMu9_1.root'
        lfn(lfnA)
        lfnA = '/store/group/B2G/SDMu9_Zmumu/Zmumu/OctX_HZZ3lepSkim_SDMu9/1eb161a436e69f7af28d18145e4ce909/3lepSkim_SDMu9_1.root'
        lfn(lfnA)
        lfnA = '/store/group/phys_higgs/meridian/HGGProd/GluGluToHToGG_M-125_8TeV-powheg-pythia6-Summer12-START53_V7D-v2/meridian/GluGlu_HToGG_M-125_8TeV-powheg-LHE_v1/GluGluToHToGG_M-125_8TeV-powheg-pythia6-Summer12-START53_V7D-v2/fb576e5b6a5810681def50b608ec31ad/Hadronizer_TuneZ2star_8TeV_Powheg_pythia_tauola_cff_py_GEN_SIM_DIGI_L1_DIGI2RAW_RAW2DIGI_L1Reco_RECO_PU_1_1_ukQ.root'
        lfn(lfnA)
        lfnA = '/store/lhe/7365/TprimeTprimeToTHTH_M-400_TuneZ2star_8TeV-madgraph_50219221.lhe'
        lfn(lfnA)
        lfnA = '/store/lhe/10860/LQToUE_BetaHalf_vector_YM-MLQ300LG0KG0.lhe.xz'
        lfn(lfnA)
        lfnA = '/store/lhe/7365/mysecondary/0001/TprimeTprimeToTHTH_M-400_TuneZ2star_8TeV-madgraph_50219221.lhe'
        lfn(lfnA)
        lfnA = '/store/lhe/10860/mysecondary/0001/LQToUE_BetaHalf_vector_YM-MLQ300LG0KG0.lhe.xz'
        lfn(lfnA)

        # All these cases should fail
        lfnA = '/storeA/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/Temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/Lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_;10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron;-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-rECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1;/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000a/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X;-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/000/000/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/000/Iamralph/000/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)

        # All these cases should fail
        lfnA = '/storeA/temp/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/Temp/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/acquisition_;10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/acquisition_10-A/MuElectron;-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/acquisition_10-A/MuElectron-10_100/RAW-rECO/vX-1/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1;/1000/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000a/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X;-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)

        lfnA = '/store/temp/user/ewv/Higgs;123/PrivateSample/v1/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/user/ewv/Higgs-123/Private;Sample/v1/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/user/ewv/Higgs-123/PrivateSample/v1;/a_X-2.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/mc/2008/2/FastSim-CSA07Electron-1203615548/B6E531DD-99E1-DC11-9FEC-001617E30D4A.root'
        self.assertRaises(AssertionError, lfn, lfnA)

        lfnA = '/store/Results/qcd/QCD_Pt80/StoreResults-Summer09-MC_31X_V3_7TeV-Jet30U-JetAODSkim-0a98be42532eba1f0545cc9b086ec3c3/QCD_Pt80/USER/StoreResults-Summer09-MC_31X_V3_7TeV-Jet30U-JetAODSkim-0a98be42532eba1f0545cc9b086ec3c3/0000/C44630AC-C0C7-DE11-AD4E-0019B9CAC0F8.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/Results/qcd123/QCD_Pt80/StoreResults-Summer09-MC_31X_V3_7TeV-Jet30U-JetAODSkim-0a98be42532eba1f0545cc9b086ec3c3/QCD_Pt80/USER/StoreResults-Summer09-MC_31X_V3_7TeV-Jet30U-JetAODSkim-0a98be42532eba1f0545cc9b086ec3c3/0000/C44630AC-C0C7-DE11-AD4E-0019B9CAC0F8.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/results/qcd/QCD_Pt80/StoreResults-Summer09-MC_31X_V3_7TeV-Jet30U-JetAODSkim-0a98be42532eba1f0545cc9b086ec3c3/QCD_Pt80/USER/StoreResults-Summer09-MC_31X_V3_7TeV-Jet30U-JetAODSkim-0a98be42532eba1f0545cc9b086ec3c3/0000a/C44630AC-C0C7-DE11-AD4E-0019B9CAC0F8.root'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/lhe/10860/11%11/1111/LQToUE_BetaHalf_vector_YM-MLQ300LG0KG0.lhe.xz'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/lhe/TprimeTprimeToTHTH_M-400_TuneZ2star_8TeV-madgraph_50219221.lhe'
        self.assertRaises(AssertionError, lfn, lfnA)
        return

    def testLFNBase(self):
        """
        _testLFNBase_

        Test the LFN Base
        """

        lfnA = '/store/temp/user/ewv/Higgs-123/PrivateSample/v1'
        lfnBase(lfnA)
        lfnA = '/store/user/ewv/Higgs-123/PrivateSample/v1'
        lfnBase(lfnA)
        lfnA = '/store/temp/group/Exotica/Higgs-123/PrivateSample/v1'
        lfnBase(lfnA)
        lfnA = '/store/user/group/Exotica/PrivateSample/v1'
        lfnBase(lfnA)
        lfnA = '/store/temp1/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        lfnBase(lfnA)
        lfnA = '/store/temp/lustre1/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        lfnBase(lfnA)

        lfnA = '/store/hidata/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        lfnBase(lfnA)
        lfnA = '/store/hidata/Run2010A/Cosmics/RECO/v4'
        lfnBase(lfnA)
        lfnA = '/store/results/qcd/StoreResults/QCD_Pt_40_2017_14TeV_612_SLHC6_patch1/USER/QCD_Pt_40_2017_14TeV_612_SLHC6_patch1_6be6d116203e430d91d7e1d6d9a88cd7-v1'
        lfnBase(lfnA)
        lfnA = '/Store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/Temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/Lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition;_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10;_100/RAW-RECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-rECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX;-1'
        self.assertRaises(AssertionError, lfn, lfnA)

        lfnA = '/Store/temp/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/Temp/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/acquisition;_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/acquisition_10-A/MuElectron-10;_100/RAW-RECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/acquisition_10-A/MuElectron-10_100/RAW-rECO/vX-1'
        self.assertRaises(AssertionError, lfn, lfnA)
        lfnA = '/store/temp/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX;-1'
        self.assertRaises(AssertionError, lfn, lfnA)

        return

    def testUserLFN(self):
        """
        _testUserLFN_

        Test the LFN checker for user LFNs that are not part of a dataset (TFiles, logs)
        """

        lfnA = '/store/temp/user/ewv/CRAB-Out/ewv_crab_EwvAnalysis_46_111109_143552/output/total_0001.json'
        userLfn(lfnA)
        lfnA = '/store/user/ewv/CRAB-Out/ewv_crab_EwvAnalysis_46_111109_143552/output/total_0001.json'
        userLfn(lfnA)
        lfnA = "/store/temp/user/ewv/CRAB-Out/ewv_crab_EwvAnalysis_45_111109_130928/output/histo_0002.root"
        userLfn(lfnA)

        return

    def testUserLFNBase(self):
        """
        _testLFNBase_

        Test the LFN Base checker for user LFNs that are not part of a dataset (TFiles, logs)
        """

        lfnA = '/store/temp/user/ewv/CRAB-Out/ewv_crab_EwvAnalysis_46_111109_143552/output'
        userLfnBase(lfnA)
        lfnA = '/store/user/ewv/CRAB-Out/ewv_crab_EwvAnalysis_46_111109_143552/output'
        userLfnBase(lfnA)

        return

    def testLFNParser(self):
        """
        _testLFNParser_

        Check and make sure that we parse LFNs correctly
        """

        lfnA = '/store/temp/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        result = parseLFN(lfnA)

        self.assertEqual(result['baseLocation'], '/store/temp')
        self.assertEqual(result['acquisitionEra'], 'acquisition_10-A')
        self.assertEqual(result['primaryDataset'], 'MuElectron-10_100')
        self.assertEqual(result['dataTier'], 'RAW-RECO')
        self.assertEqual(result['processingVersion'], 'vX-1')
        self.assertEqual(result['lfnCounter'], '1000')
        self.assertEqual(result['filename'], 'a_X-2.root')

        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1/1000/a_X-2.root'
        result = parseLFN(lfnA)

        self.assertEqual(result['baseLocation'], '/store/temp/lustre')
        self.assertEqual(result['acquisitionEra'], 'acquisition_10-A')
        self.assertEqual(result['primaryDataset'], 'MuElectron-10_100')
        self.assertEqual(result['dataTier'], 'RAW-RECO')
        self.assertEqual(result['processingVersion'], 'vX-1')
        self.assertEqual(result['lfnCounter'], '1000')
        self.assertEqual(result['filename'], 'a_X-2.root')

        lfnA = '/store/temp/user/ewv/Higgs-123/PrivateSample/v1/1000/a_X-2.root'
        result = parseLFN(lfnA)

        self.assertEqual(result['baseLocation'], '/store/temp/user')
        self.assertEqual(result['primaryDataset'], 'Higgs-123')
        self.assertEqual(result['secondaryDataset'], 'PrivateSample')
        self.assertEqual(result['processingVersion'], 'v1')
        self.assertEqual(result['filename'], 'a_X-2.root')

        lfnA = '/store/user/ewv/Higgs-123/PrivateSample/v1/1000/a_X-2.root'
        result = parseLFN(lfnA)

        self.assertEqual(result['baseLocation'], '/store/user')
        self.assertEqual(result['primaryDataset'], 'Higgs-123')
        self.assertEqual(result['secondaryDataset'], 'PrivateSample')
        self.assertEqual(result['processingVersion'], 'v1')
        self.assertEqual(result['filename'], 'a_X-2.root')

        lfnA = '/store/group/Exotica/Higgs-123/PrivateSample/v1/1000/a_X-2.root'
        result = parseLFN(lfnA)

        self.assertEqual(result['baseLocation'], '/store/group')
        self.assertEqual(result['primaryDataset'], 'Higgs-123')
        self.assertEqual(result['secondaryDataset'], 'PrivateSample')
        self.assertEqual(result['processingVersion'], 'v1')
        self.assertEqual(result['filename'], 'a_X-2.root')

        return

    def testLFNBaseParser(self):
        """
        _testLFNBaseParser_

        Test the parsing for LFN Base
        """

        lfnA = '/store/temp/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        result = parseLFNBase(lfnA)

        self.assertEqual(result['baseLocation'], '/store/temp')
        self.assertEqual(result['acquisitionEra'], 'acquisition_10-A')
        self.assertEqual(result['primaryDataset'], 'MuElectron-10_100')
        self.assertEqual(result['dataTier'], 'RAW-RECO')
        self.assertEqual(result['processingVersion'], 'vX-1')

        lfnA = '/store/temp/lustre/acquisition_10-A/MuElectron-10_100/RAW-RECO/vX-1'
        result = parseLFNBase(lfnA)

        self.assertEqual(result['baseLocation'], '/store/temp/lustre')
        self.assertEqual(result['acquisitionEra'], 'acquisition_10-A')
        self.assertEqual(result['primaryDataset'], 'MuElectron-10_100')
        self.assertEqual(result['dataTier'], 'RAW-RECO')
        self.assertEqual(result['processingVersion'], 'vX-1')

        lfnA = '/store/temp/user/ewv/Higgs-123/PrivateSample/v1'
        result = parseLFNBase(lfnA)

        self.assertEqual(result['baseLocation'], '/store/temp/user')
        self.assertEqual(result['primaryDataset'], 'Higgs-123')
        self.assertEqual(result['secondaryDataset'], 'PrivateSample')
        self.assertEqual(result['processingVersion'], 'v1')

        lfnA = '/store/user/ewv/Higgs-123/PrivateSample/v1'
        result = parseLFNBase(lfnA)

        self.assertEqual(result['baseLocation'], '/store/user')
        self.assertEqual(result['primaryDataset'], 'Higgs-123')
        self.assertEqual(result['secondaryDataset'], 'PrivateSample')
        self.assertEqual(result['processingVersion'], 'v1')

        lfnA = '/store/group/Exotica/Higgs-123/PrivateSample/v1'
        result = parseLFNBase(lfnA)

        self.assertEqual(result['baseLocation'], '/store/group')
        self.assertEqual(result['primaryDataset'], 'Higgs-123')
        self.assertEqual(result['secondaryDataset'], 'PrivateSample')
        self.assertEqual(result['processingVersion'], 'v1')

        return

    def testSanitizeURL(self):
        proto = "http"
        host = "test.com"
        user = "abc"
        passwd = "^cba$"
        port = "9999"
        url = "%s://%s:%s@%s:%s" % (proto, user, passwd, host, port)
        urlDict = sanitizeURL(url)
        self.assertEqual(urlDict['url'], "%s://%s:%s" % (proto, host, port))
        self.assertEqual(urlDict['username'], user)
        self.assertEqual(urlDict['password'], passwd)

        noPassURL = "http://test.com"
        urlDict = sanitizeURL(noPassURL)
        self.assertEqual(urlDict['url'], noPassURL)
        self.assertEqual(urlDict['username'], None)
        self.assertEqual(urlDict['password'], None)

    def testReplaceToSantizeURL(self):
        """
        Checks that a URI containing a username password has that information
        removed when an error occurs.
        """
        dbUrl = "mysql://DBUSER:DBSPASSWORD@localhost:80808/WMAgentDB"
        errorMsg = "DB failure: %s" % (dbUrl)
        dbUrl2 = "mysql://localhost:80808/WMAgentDB"
        sanitizedError = "DB failure: %s" % (dbUrl2)
        self.assertEqual(replaceToSantizeURL(errorMsg), sanitizedError)

        dbUrl = "MySql://DBUSER:DBSPASSWORD@localhost:80808/WMAgentDB"
        errorMsg = "DB failure: %s" % (dbUrl)
        dbUrl2 = "MySql://localhost:80808/WMAgentDB"
        sanitizedError = "DB failure: %s" % (dbUrl2)
        self.assertEqual(replaceToSantizeURL(errorMsg), sanitizedError)

        dbUrl = "mysql://localhost:80808/WMAgentDB"
        errorMsg = "DB failure: %s" % (dbUrl)
        dbUrl2 = "mysql://localhost:80808/WMAgentDB"
        sanitizedError = "DB failure: %s" % (dbUrl2)
        self.assertEqual(replaceToSantizeURL(errorMsg), sanitizedError)

        dbUrl = "http://DBUSER:DBSPASSWORD@localhost:80808/WMAgentDB"
        errorMsg = "DB failure: %s" % (dbUrl)
        dbUrl2 = "http://localhost:80808/WMAgentDB"
        sanitizedError = "DB failure: %s" % (dbUrl2)
        self.assertEqual(replaceToSantizeURL(errorMsg), sanitizedError)

        dbUrl = "HtTp://DBUSER:DBSPASSWORD@localhost:80808/WMAgentDB"
        errorMsg = "DB failure: %s" % (dbUrl)
        dbUrl2 = "HtTp://localhost:80808/WMAgentDB"
        sanitizedError = "DB failure: %s" % (dbUrl2)
        self.assertEqual(replaceToSantizeURL(errorMsg), sanitizedError)

        dbUrl = "http://localhost:80808/WMAgentDB"
        errorMsg = "DB failure: %s" % (dbUrl)
        dbUrl2 = "http://localhost:80808/WMAgentDB"
        sanitizedError = "DB failure: %s" % (dbUrl2)
        self.assertEqual(replaceToSantizeURL(errorMsg), sanitizedError)

    def testSplitCouchServiceURL(self):

        urlSplit = splitCouchServiceURL("https://cmsweb-dev.cern.ch:8888/workqueue")
        self.assertEqual("https://cmsweb-dev.cern.ch:8888", urlSplit[0])
        self.assertEqual("workqueue", urlSplit[1])

        urlSplit = splitCouchServiceURL("https://cmsweb-dev.cern.ch/couchdb/workqueue")
        self.assertEqual("https://cmsweb-dev.cern.ch/couchdb", urlSplit[0])
        self.assertEqual("workqueue", urlSplit[1])

        return

    def testGlobalTag(self):
        """
        Test and check with some global tags.

        """

        gTag = 'START_V2::ALL'
        globalTag(gTag)
        gTag = 'START_V2;;ALL'
        self.assertRaises(AssertionError, globalTag, gTag)

        return

    def testUrlValidation(self):
        """
        Test the validateUrl function for some use case of DBS 3
        """
        # Good http(s) urls
        for url in ['https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader',
                    'https://mydbs.mydomain.de:8443',
                    'http://localhost',
                    'http://192.168.1.1',
                    'https://192.168.1.1:443',
                    'http://[2001:0db8:85a3:08d3:1319:8a2e:0370:7344]/',
                    'http://2001:0db8:85a3:08d3:1319:8a2e:0370:7344/',
                    'http://[2001:0db8:85a3:08d3:1319:8a2e:0370:7344]:8443/']:
            validateUrl(url)

        # Bad http(s) urls
        for url in ['ftp://dangerous.download.this',
                    'https:/notvalid.url',
                    'http://-NiceTry',
                    'https://NiceTry; DROP Table;--',
                    'http://123123104122',
                    'http://.www.google.com',
                    'http://[2001:0db8:85a3:08d3:1319:8a2z:0370:7344]/', ]:
            self.assertRaises(AssertionError, validateUrl, url)

    def testPrimaryDatasetType(self):
        self.assertRaises(AssertionError, primaryDatasetType, "MC")
        self.assertTrue(primaryDatasetType("mc"), "mc should be allowed")
        self.assertTrue(primaryDatasetType("data"), "data should be allowed")
        self.assertTrue(primaryDatasetType("cosmic"), "data should be allowed")
        self.assertTrue(primaryDatasetType("test"), "test should be allowed")

    def testPhysicsGroup(self):
        """
        _testPhysicsGroup_

        Test a few use cases for PhysicsGroup
        """
        self.assertRaises(AssertionError, physicsgroup, '')
        self.assertRaises(AssertionError, physicsgroup, 'A-30-length-Str_that_is_not_allowed')

        self.assertTrue(physicsgroup('1'))
        self.assertTrue(physicsgroup('A'))
        self.assertTrue(physicsgroup('_'))
        self.assertTrue(physicsgroup('-'))
        self.assertTrue(physicsgroup('FacOps'))
        self.assertTrue(physicsgroup('Heavy-Ions'))
        self.assertTrue(physicsgroup('Heavy_Ions'))
        self.assertTrue(physicsgroup('Tracker-POG'))
        self.assertTrue(physicsgroup('Trigger'))
        return

    def testGetStringsBetween(self):
        start = '<a n="MachineAttrGLIDEIN_CMSSite0"><s>'
        end = '</s></a>'
        source = '<a n="MachineAttrGLIDEIN_CMSSite0"><s>T2_US_Florida</s></a>'
        result = getStringsBetween(start, end, source)
        self.assertEqual(result, 'T2_US_Florida')

    def testGetIterMatchObjectOnRegex(self):
        logPath = os.path.join(getTestBase(), "WMCore_t/test_empty.log")
        for mo in getIterMatchObjectOnRegexp(logPath, WMEXCEPTION_REGEXP):
            pass

        count = 0
        ecount = 0
        logPath = os.path.join(getTestBase(), "WMCore_t/test_condor.log")
        for mo in getIterMatchObjectOnRegexp(logPath, WMEXCEPTION_REGEXP):
            errMsg = mo.group("WMException")
            if errMsg:
                count += 1
            error = mo.group("ERROR")
            if error:
                ecount += 1
        self.assertEqual(count, 4)
        self.assertEqual(ecount, 1)

        rcount = 0
        scount = 0
        for mo in getIterMatchObjectOnRegexp(logPath, CONDOR_LOG_FILTER_REGEXP):
            if mo.group("Reason"):
                reason = mo.group("Reason")
                rcount += 1
            if mo.group("Site"):
                site = mo.group("Site")
                scount += 1

        self.assertEqual(rcount, 1)
        self.assertEqual(scount, 2)
        self.assertEqual(site, 'T1_US_FNAL')
        self.assertEqual(reason, 'via condor_rm (by user cmst1)')

    def testTaskStepName(self):
        """
        Test some task and step names
        """
        self.assertTrue(taskStepName("T"))
        self.assertTrue(taskStepName("TaskName-Test_Num1"))
        self.assertTrue(taskStepName("this_is-a-step-name"))
        self.assertTrue(taskStepName("Test123-456_789"))
        self.assertTrue(taskStepName("t" * 50))

        # now bad names
        self.assertRaises(AssertionError, taskStepName, "")
        self.assertRaises(AssertionError, taskStepName, "1Task")
        self.assertRaises(AssertionError, taskStepName, "-Task_testName")
        self.assertRaises(AssertionError, taskStepName, "_Task_testName")
        self.assertRaises(AssertionError, taskStepName, "Task@testName")
        self.assertRaises(AssertionError, taskStepName, "t" * 51)

    def testGpuParameters(self):
        """
        Test the 'GPUParams' spec parameter via 'gpuParameters' function
        """
        # unsupported values or incomplete information
        self.assertRaises(AssertionError, gpuParameters, "")
        self.assertRaises(AssertionError, gpuParameters, {})
        self.assertRaises(AssertionError, gpuParameters, json.dumps(""))
        self.assertRaises(AssertionError, gpuParameters, json.dumps([]))
        self.assertRaises(AssertionError, gpuParameters, json.dumps(1))
        self.assertRaises(AssertionError, gpuParameters, json.dumps({}))
        # one mandatory missing
        data = {"GPUMemoryMB": 123, "CUDARuntime": "11.2"}
        self.assertRaises(AssertionError, gpuParameters, json.dumps(data))
        # one unknown argument missing
        data = {"GPUMemoryMB": 123, "CUDARuntime": "11.2", "CUDACapabilities": ["11.2"], "BAD_KEY": 123}
        self.assertRaises(AssertionError, gpuParameters, json.dumps(data))
        # only optional arguments, regardless of their values
        data = {"GPUName": "NVidia 970M GTX", "CUDADriverVersion": "11.2", "CUDARuntimeVersion": 123}
        self.assertRaises(AssertionError, gpuParameters, json.dumps(data))

        # valid and well supported cases
        self.assertTrue(gpuParameters(json.dumps(None)))
        # all mandatory arguments
        data = {"GPUMemoryMB": 123, "CUDARuntime": "11.2", "CUDACapabilities": ["11.2"]}
        self.assertTrue(gpuParameters(json.dumps(data)))
        # all mandatory arguments plus one optional argument
        data = {"GPUMemoryMB": 123, "CUDARuntime": "11.2", "CUDACapabilities": ["11.2"],
                "GPUName": "Nvidia"}
        self.assertTrue(gpuParameters(json.dumps(data)))
        # all mandatory arguments plus two optional argument
        data = {"GPUMemoryMB": 123, "CUDARuntime": "11.2", "CUDACapabilities": ["11.2"],
                "GPUName": "Nvidia", "CUDADriverVersion": "11.2"}
        self.assertTrue(gpuParameters(json.dumps(data)))
        # all mandatory and all optional arguments
        data = {"GPUMemoryMB": 123, "CUDARuntime": "11.2", "CUDACapabilities": ["11.2"],
                "GPUName": "Nvidia", "CUDADriverVersion": "11.2", "CUDARuntimeVersion": "11.2.03"}
        self.assertTrue(gpuParameters(json.dumps(data)))

    def testGpuInternalParameters(self):
        """
        Test the inner key/value pairs of 'GPUParams' spec parameter,
        via '_gpuInternalParameters' private function
        """
        # test "GPUMemoryMB" validation
        self.assertRaises(AssertionError, _gpuInternalParameters, {"GPUMemoryMB": "123"})
        self.assertRaises(AssertionError, _gpuInternalParameters, {"GPUMemoryMB": [123]})
        self.assertRaises(AssertionError, _gpuInternalParameters, {"GPUMemoryMB": None})
        self.assertRaises(AssertionError, _gpuInternalParameters, {"GPUMemoryMB": 100.5})
        self.assertRaises(AssertionError, _gpuInternalParameters, {"GPUMemoryMB": 0})
        # now pass "GPUMemoryMB", but fail "CUDACapabilities"
        self.assertRaises(AssertionError, _gpuInternalParameters, {"GPUMemoryMB": 1, "CUDACapabilities": 123})
        self.assertRaises(AssertionError, _gpuInternalParameters, {"GPUMemoryMB": 1, "CUDACapabilities": "123"})
        self.assertRaises(AssertionError, _gpuInternalParameters, {"GPUMemoryMB": 1, "CUDACapabilities": None})
        self.assertRaises(AssertionError, _gpuInternalParameters, {"GPUMemoryMB": 1, "CUDACapabilities": []})
        self.assertRaises(AssertionError, _gpuInternalParameters, {"GPUMemoryMB": 1, "CUDACapabilities": ["1.2", 1.2]})
        # now pass "GPUMemoryMB" and "CUDACapabilities", but fail "CUDARuntime"
        self.assertRaises(AssertionError, _gpuInternalParameters,
                          {"GPUMemoryMB": 1, "CUDACapabilities": ["1.2"], "CUDARuntime": 123})
        self.assertRaises(AssertionError, _gpuInternalParameters,
                          {"GPUMemoryMB": 1, "CUDACapabilities": ["1.2"], "CUDARuntime": [123]})
        self.assertRaises(AssertionError, _gpuInternalParameters,
                          {"GPUMemoryMB": 1, "CUDACapabilities": ["1.2"], "CUDARuntime": None})
        self.assertRaises(AssertionError, _gpuInternalParameters,
                          {"GPUMemoryMB": 1, "CUDACapabilities": ["1.2"], "CUDARuntime": "123"})
        # now test valid values for all 3 mandatory arguments: GPUMemoryMB, CUDACapabilities, CUDARuntime
        self.assertTrue(_gpuInternalParameters({"GPUMemoryMB": 1, "CUDACapabilities": ["1.2"], "CUDARuntime": "2.3"}))
        self.assertTrue(_gpuInternalParameters({"GPUMemoryMB": 12345, "CUDACapabilities": ["1.2", "2.3", "2.3.4"],
                                                "CUDARuntime": "2.3"}))
        self.assertTrue(_gpuInternalParameters({"GPUMemoryMB": 12345, "CUDACapabilities": ["1.2", "2.3", "2.3.4"],
                                                "CUDARuntime": "2.3.4"}))

        goodParams = {"GPUMemoryMB": 12345, "CUDACapabilities": ["1.2"], "CUDARuntime": "2.3.4"}
        # test "GPUName" validation
        goodParams.update({"GPUName": 123})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"GPUName": None})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"GPUName": []})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"GPUName": 123})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"GPUName": 101*"a"})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        # now pass "GPUName", and test "CUDADriverVersion" parameter
        goodParams = {"GPUMemoryMB": 12345, "CUDACapabilities": ["1.2"], "CUDARuntime": "2.3.4",
                      "GPUName": "123"}
        goodParams.update({"CUDADriverVersion": 123})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"CUDADriverVersion": None})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"CUDADriverVersion": []})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"CUDADriverVersion": "123"})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"CUDADriverVersion": "1.2.3.4"})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"CUDADriverVersion": 101*"1"})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        # now pass "GPUName" and "CUDADriverVersion", and test "CUDARuntimeVersion" parameter
        goodParams = {"GPUMemoryMB": 12345, "CUDACapabilities": ["1.2"], "CUDARuntime": "2.3.4",
                      "GPUName": "123", "CUDADriverVersion": "1.2.3"}
        goodParams.update({"CUDARuntimeVersion": 123})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"CUDARuntimeVersion": None})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"CUDARuntimeVersion": []})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"CUDARuntimeVersion": "123"})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"CUDARuntimeVersion": "1.2.3.4"})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)
        goodParams.update({"CUDARuntimeVersion": 101*"1"})
        self.assertRaises(AssertionError, _gpuInternalParameters, goodParams)

        # finally, test good values for mandatory + optional parameters
        # now test valid values for all 3 mandatory arguments: GPUMemoryMB, CUDACapabilities, CUDARuntime
        self.assertTrue(_gpuInternalParameters({"GPUMemoryMB": 1, "CUDACapabilities": ["1.2"],
                                                "CUDARuntime": "2.3", "GPUName": "123",
                                                "CUDADriverVersion": "1.2.3", "CUDARuntimeVersion": "1.2.03"}))
        self.assertTrue(_gpuInternalParameters({"GPUMemoryMB": 12345, "CUDACapabilities": ["1.2", "2.3", "2.3.4"],
                                                "CUDARuntime": "2.3"}))
        self.assertTrue(_gpuInternalParameters({"GPUMemoryMB": 12345, "CUDACapabilities": ["1.2", "2.3", "2.3.4"],
                                                "CUDARuntime": "2.3.4"}))

    def testSubRequestType(self):
        """
        Test some task and step names
        """
        # valid values
        for cand in ["MC", "ReDigi", "Pilot", "RelVal", "HIRelVal", "ReReco", ""]:
            self.assertTrue(subRequestType(cand))

        # now invalid ones
        self.assertRaises(AssertionError, subRequestType, None)
        self.assertRaises(AssertionError, subRequestType, "test")
        self.assertRaises(AssertionError, subRequestType, ["blah"])
        self.assertRaises(AssertionError, subRequestType, 1)


if __name__ == "__main__":
    unittest.main()



from WMCore.WebTools.RESTModel import RESTModel, restexpose
from cherrypy import HTTPError
import unittest, logging, json
from WMQuality.WebTools.RESTServerSetup import cherrypySetup, DefaultConfig
from WMQuality.WebTools.RESTClientAPI import makeRequest, methodTest

class REST_Exceptions_t(RESTModel):
    def __init__(self, config):
        '''
        Initialise the RESTModel and add some methods to it.
        '''
        RESTModel.__init__(self, config)

        self.methods['GET'] = {}
        self.methods['GET']['generic_exception'] = {'args': [],
                                         'call': self.generic_exception,
                                         'version': 1}

        self._addMethod('GET', 'specific_400_exception', self.specific_400_exception)
        self._addMethod('GET', 'specific_500_exception', self.specific_500_exception)
        self._addMethod('GET', 'specific_404_exception', self.specific_404_exception)
        self._addMethod('GET', 'not_serialisable', self.not_serialisable)

    @restexpose
    def generic_exception(self):
        """
        Raise an exception - this will result in a 500 Server Error from the RESTAPI
        """
        assert 1 == 2, "1 does not equal 2"

    def specific_400_exception(self):
        """
        Raise an HTTP Error, this will be preserved and propagated to the client
        """
        raise HTTPError(400, 'I threw a 400')

    def specific_500_exception(self):
        """
        Raise an HTTP Error, this will be preserved and propagated to the client
        """
        raise HTTPError(500, 'I threw a 500')

    def specific_404_exception(self):
        """
        Raise an HTTP Error, this will be preserved and propagated to the client
        """
        raise HTTPError(404, 'I threw a 404')

    def not_serialisable(self):
        """
        Raise an exception in the formatter (complex numbers aren't json serialisable
        by default), this is caught and turned into a 500 Server Error by the RESTAPI
        """
        return complex(1,2)

test_config = DefaultConfig('WMCore_t.WebTools_t.REST_Exceptions_t')
test_config.Webtools.access_log_level = logging.WARNING
test_config.Webtools.error_log_level = logging.WARNING

from WMQuality.WebTools.RESTBaseUnitTest import RESTBaseUnitTest
# Disabling tests because the decorator doesn't work right
class RESTTestFAIL():

    def setUp(self):
        self.config = test_config
        self.dasFlag = False
        self.urlbase = self.config.getServerUrl()

    def tearDown(self):
        self.dasFlag = None
        self.urlbase = None

    @cherrypySetup(test_config)
    def testGenericException(self):
        """
        Method will raise an AssertionError and return 500
        """
        url = self.urlbase + 'generic_exception'
        response, expires = methodTest('GET', url, output={'code':500})
        assert json.loads(response)['message'] == "Server Error", 'got: %s' % json.loads(response)['message']
        assert json.loads(response)['type'] == "AssertionError", 'got: %s' % json.loads(response)['type']

    @cherrypySetup(test_config)
    def testSpecific400Exception(self):
        """
        Method will raise an HTTPError and return 400
        """
        url = self.urlbase + 'specific_400_exception'
        response, expires = methodTest('GET', url, output={'code':400})
        assert json.loads(response)['message'] == "I threw a 400", 'got: %s' % json.loads(response)['message']

    @cherrypySetup(test_config)
    def testSpecific404Exception(self):
        """
        Method will raise an HTTPError and return 404
        """
        url = self.urlbase + 'specific_404_exception'
        response, expires = methodTest('GET', url, output={'code':404})
        assert json.loads(response)['message'] == "I threw a 404", 'got: %s' % json.loads(response)['message']

    @cherrypySetup(test_config)
    def testSpecific500Exception(self):
        """
        Method will raise an HTTPError and return 500
        """
        url = self.urlbase + 'specific_500_exception'
        response, expires = methodTest('GET', url, output={'code':500})
        assert json.loads(response)['message'] == "I threw a 500", 'got: %s' % json.loads(response)['message']

    @cherrypySetup(test_config)
    def testNotSerialisableException(self):
        """
        Method will raise an EncodeError and return 500
        """
        url = self.urlbase + 'not_serialisable'
        response, expires = methodTest('GET', url, output={'code':500})
        assert json.loads(response)['message'] == "Server Error", 'got: %s' % json.loads(response)['message']
        assert json.loads(response)['type'] == "TypeError", 'got: %s' % json.loads(response)['type']

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_LoadFromTask_

MySQL implementation of Workflow.LoadFromTask
"""

__all__ = []



from WMCore.Database.DBFormatter import DBFormatter

class LoadFromTask(DBFormatter):
    sql = """SELECT wmbs_workflow.id, wmbs_workflow.spec, wmbs_workflow.name, wmbs_users.cert_dn as owner, wmbs_workflow.task
             FROM wmbs_workflow
             INNER JOIN wmbs_users ON
               wmbs_workflow.owner = wmbs_users.id
             WHERE wmbs_workflow.task = :task"""

    def formatDict(self, result):
        """
        _formatDict_

        Cast the id attribute to an int because the DBFormatter turns everything
        into strings.
        """
        tempResults = DBFormatter.formatDict(self, result)

        formattedResults = []
        for tempResult in tempResults:
            tempResult["id"] = int(tempResult["id"])
            formattedResults.append(tempResult)

        return formattedResults

    def execute(self, task = None, conn = None, transaction = False):
        result = self.dbi.processData(self.sql, {"task": task},
                         conn = conn, transaction = transaction)
        return self.formatDict(result)



#!/usr/bin/env python
"""
_DBSBufferFile_t_

Unit tests for the DBSBufferFile class.
"""

from builtins import int

import threading
import unittest

from WMComponent.DBS3Buffer.DBSBufferBlock import DBSBufferBlock
from WMComponent.DBS3Buffer.DBSBufferFile import DBSBufferFile
from WMComponent.DBS3Buffer.DBSBufferUtil import DBSBufferUtil
from WMCore.DAOFactory import DAOFactory
from WMCore.DataStructs.Run import Run
from WMQuality.TestInit import TestInit


class DBSBufferFileTest(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Setup the database and logging connection.  Try to create all of the
        DBSBuffer tables.  Also add some dummy locations.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMComponent.DBS3Buffer"],
                                useDefault=False)

        myThread = threading.currentThread()
        self.daoFactory = DAOFactory(package="WMComponent.DBS3Buffer",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)

        locationAction = self.daoFactory(classname="DBSBufferFiles.AddLocation")
        locationAction.execute(siteName="se1.cern.ch")
        locationAction.execute(siteName="se1.fnal.gov")

    def tearDown(self):
        """
        _tearDown_

        Drop all the DBSBuffer tables.
        """
        self.testInit.clearDatabase()

    def testCreateDeleteExists(self):
        """
        _testCreateDeleteExists_

        Test the create(), delete() and exists() methods of the file class
        by creating and deleting a file.  The exists() method will be
        called before and after creation and after deletion.
        """
        testFile = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10)
        testFile.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                              appFam="RECO", psetHash="GIBBERISH",
                              configContent="MOREGIBBERISH")
        testFile.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")

        assert testFile.exists() == False, \
            "ERROR: File exists before it was created"

        testFile.addRun(Run(1, *[45]))
        testFile.create()

        assert testFile.exists() > 0, \
            "ERROR: File does not exist after it was created"

        testFile.delete()

        assert testFile.exists() == False, \
            "ERROR: File exists after it has been deleted"
        return

    def testCreateTransaction(self):
        """
        _testCreateTransaction_

        Begin a transaction and then create a file in the database.  Afterwards,
        rollback the transaction.  Use the File class's exists() method to
        to verify that the file doesn't exist before it was created, exists
        after it was created and doesn't exist after the transaction was rolled
        back.
        """
        myThread = threading.currentThread()
        myThread.transaction.begin()

        testFile = DBSBufferFile(lfn="/this/is/a/lfn", size=1024,
                                 events=10)
        testFile.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                              appFam="RECO", psetHash="GIBBERISH",
                              configContent="MOREGIBBERISH")
        testFile.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")

        assert testFile.exists() == False, \
            "ERROR: File exists before it was created"
        testFile.addRun(Run(1, *[45]))
        testFile.create()

        assert testFile.exists() > 0, \
            "ERROR: File does not exist after it was created"

        myThread.transaction.rollback()

        assert testFile.exists() == False, \
            "ERROR: File exists after transaction was rolled back."
        return

    def testDeleteTransaction(self):
        """
        _testDeleteTransaction_

        Create a file and commit it to the database.  Start a new transaction
        and delete the file.  Rollback the transaction after the file has been
        deleted.  Use the file class's exists() method to verify that the file
        does not exist after it has been deleted but does exist after the
        transaction is rolled back.
        """
        testFile = DBSBufferFile(lfn="/this/is/a/lfn", size=1024,
                                 events=10)
        testFile.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                              appFam="RECO", psetHash="GIBBERISH",
                              configContent="MOREGIBBERISH")
        testFile.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")

        assert testFile.exists() == False, \
            "ERROR: File exists before it was created"

        testFile.addRun(Run(1, *[45]))
        testFile.create()

        assert testFile.exists() > 0, \
            "ERROR: File does not exist after it was created"

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testFile.delete()

        assert testFile.exists() == False, \
            "ERROR: File exists after it has been deleted"

        myThread.transaction.rollback()

        assert testFile.exists() > 0, \
            "ERROR: File does not exist after transaction was rolled back."
        return

    def testGetParentLFNs(self):
        """
        _testGetParentLFNs_

        Create three files and set them to be parents of a fourth file.  Check
        to make sure that getParentLFNs() on the child file returns the correct
        LFNs.
        """
        testFileParentA = DBSBufferFile(lfn="/this/is/a/parent/lfnA", size=1024,
                                        events=20)
        testFileParentA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                     appFam="RECO", psetHash="GIBBERISH",
                                     configContent="MOREGIBBERISH")
        testFileParentA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileParentA.addRun(Run(1, *[45]))

        testFileParentB = DBSBufferFile(lfn="/this/is/a/parent/lfnB", size=1024,
                                        events=20)
        testFileParentB.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                     appFam="RECO", psetHash="GIBBERISH",
                                     configContent="MOREGIBBERISH")
        testFileParentB.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileParentB.addRun(Run(1, *[45]))

        testFileParentC = DBSBufferFile(lfn="/this/is/a/parent/lfnC", size=1024,
                                        events=20)
        testFileParentC.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                     appFam="RECO", psetHash="GIBBERISH",
                                     configContent="MOREGIBBERISH")
        testFileParentC.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileParentC.addRun(Run(1, *[45]))

        testFileParentA.create()
        testFileParentB.create()
        testFileParentC.create()

        testFile = DBSBufferFile(lfn="/this/is/a/lfn", size=1024,
                                 events=10)
        testFile.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                              appFam="RECO", psetHash="GIBBERISH",
                              configContent="MOREGIBBERISH")
        testFile.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFile.addRun(Run(1, *[45]))
        testFile.create()

        testFile.addParents([testFileParentA["lfn"], testFileParentB["lfn"],
                             testFileParentC["lfn"]])

        parentLFNs = testFile.getParentLFNs()

        assert len(parentLFNs) == 3, \
            "ERROR: Child does not have the right amount of parents"

        goldenLFNs = ["/this/is/a/parent/lfnA",
                      "/this/is/a/parent/lfnB",
                      "/this/is/a/parent/lfnC"]
        for parentLFN in parentLFNs:
            assert parentLFN in goldenLFNs, \
                "ERROR: Unknown parent lfn"
            goldenLFNs.remove(parentLFN)

        testFile.delete()
        testFileParentA.delete()
        testFileParentB.delete()
        testFileParentC.delete()
        return

    def testLoad(self):
        """
        _testLoad_

        Test the loading of file meta data using the ID of a file and the
        LFN of a file.
        """
        checksums = {"adler32": "adler32", "cksum": "cksum", "md5": "md5"}
        testFileA = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10,
                                  checksums=checksums)
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.create()

        testFileB = DBSBufferFile(lfn=testFileA["lfn"])
        testFileB.load()
        testFileC = DBSBufferFile(id=testFileA["id"])
        testFileC.load()

        assert testFileA == testFileB, \
            "ERROR: File load by LFN didn't work"

        assert testFileA == testFileC, \
            "ERROR: File load by ID didn't work"

        assert isinstance(testFileB["id"], int), \
            "ERROR: File id is not an integer type."
        assert isinstance(testFileB["size"], int), \
            "ERROR: File size is not an integer type."
        assert isinstance(testFileB["events"], int), \
            "ERROR: File events is not an integer type."

        assert isinstance(testFileC["id"], int), \
            "ERROR: File id is not an integer type."
        assert isinstance(testFileC["size"], int), \
            "ERROR: File size is not an integer type."
        assert isinstance(testFileC["events"], int), \
            "ERROR: File events is not an integer type."

        testFileA.delete()
        return

    def testFilesize(self):
        """
        _testFilesize_

        Test storing and loading the file information from dbsbuffer_file.
        Make sure filesize can be bigger than 32 bits
        """
        checksums = {"adler32": "adler32", "cksum": "cksum"}
        testFileA = DBSBufferFile(lfn="/this/is/a/lfn", size=3221225472, events=1500000,
                                  checksums=checksums)
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_7_6_0",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.create()

        testFileB = DBSBufferFile(lfn=testFileA["lfn"])
        testFileB.load()
        self.assertEqual(testFileB["size"], 3221225472, "Error: the filesize should be 3GB")
        self.assertEqual(testFileB["events"], 1500000, "Error: the number of events should be 1.5M")

    def testAddChild(self):
        """
        _testAddChild_

        Add a child to some parent files and make sure that all the parentage
        information is loaded/stored correctly from the database.
        """
        testFileParentA = DBSBufferFile(lfn="/this/is/a/parent/lfnA", size=1024,
                                        events=20)
        testFileParentA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                     appFam="RECO", psetHash="GIBBERISH",
                                     configContent="MOREGIBBERISH")
        testFileParentA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")

        testFileParentA.addRun(Run(1, *[45]))
        testFileParentB = DBSBufferFile(lfn="/this/is/a/parent/lfnB", size=1024,
                                        events=20)
        testFileParentB.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                     appFam="RECO", psetHash="GIBBERISH",
                                     configContent="MOREGIBBERISH")
        testFileParentB.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileParentB.addRun(Run(1, *[45]))
        testFileParentA.create()
        testFileParentB.create()

        testFileA = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10)
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testFileParentA.addChildren("/this/is/a/lfn")
        testFileParentB.addChildren("/this/is/a/lfn")

        testFileB = DBSBufferFile(id=testFileA["id"])
        testFileB.load(parentage=1)

        goldenFiles = [testFileParentA, testFileParentB]
        for parentFile in testFileB["parents"]:
            assert parentFile in goldenFiles, \
                "ERROR: Unknown parent file"
            goldenFiles.remove(parentFile)

        assert len(goldenFiles) == 0, \
            "ERROR: Some parents are missing"
        return

    def testAddChildTransaction(self):
        """
        _testAddChildTransaction_

        Add a child to some parent files and make sure that all the parentage
        information is loaded/stored correctly from the database.  Rollback the
        addition of one of the childs and then verify that it does in fact only
        have one parent.
        """
        testFileParentA = DBSBufferFile(lfn="/this/is/a/parent/lfnA", size=1024,
                                        events=20)
        testFileParentA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                     appFam="RECO", psetHash="GIBBERISH",
                                     configContent="MOREGIBBERISH")
        testFileParentA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileParentA.addRun(Run(1, *[45]))

        testFileParentB = DBSBufferFile(lfn="/this/is/a/parent/lfnB", size=1024,
                                        events=20)
        testFileParentB.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                     appFam="RECO", psetHash="GIBBERISH",
                                     configContent="MOREGIBBERISH")
        testFileParentB.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileParentB.addRun(Run(1, *[45]))
        testFileParentA.create()
        testFileParentB.create()

        testFileA = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10)
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testFileParentA.addChildren("/this/is/a/lfn")

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testFileParentB.addChildren("/this/is/a/lfn")

        testFileB = DBSBufferFile(id=testFileA["id"])
        testFileB.load(parentage=1)

        goldenFiles = [testFileParentA, testFileParentB]
        for parentFile in testFileB["parents"]:
            assert parentFile in goldenFiles, \
                "ERROR: Unknown parent file"
            goldenFiles.remove(parentFile)

        assert len(goldenFiles) == 0, \
            "ERROR: Some parents are missing"

        myThread.transaction.rollback()
        testFileB.load(parentage=1)

        goldenFiles = [testFileParentA]
        for parentFile in testFileB["parents"]:
            assert parentFile in goldenFiles, \
                "ERROR: Unknown parent file"
            goldenFiles.remove(parentFile)

        assert len(goldenFiles) == 0, \
            "ERROR: Some parents are missing"

        return

    def testSetLocation(self):
        """
        _testSetLocation_

        Create a file and add a couple locations.  Load the file from the
        database to make sure that the locations were set correctly.
        """
        testFileA = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10)
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testFileA.setLocation(["se1.fnal.gov", "se1.cern.ch"])
        testFileA.setLocation(["bunkse1.fnal.gov", "bunkse1.cern.ch"],
                              immediateSave=False)

        testFileB = DBSBufferFile(id=testFileA["id"])
        testFileB.load()

        goldenLocations = ["se1.fnal.gov", "se1.cern.ch"]

        for location in testFileB["locations"]:
            assert location in goldenLocations, \
                "ERROR: Unknown file location"
            goldenLocations.remove(location)

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing"
        return

    def testSetLocationTransaction(self):
        """
        _testSetLocationTransaction_

        Create a file at specific locations and commit everything to the
        database.  Reload the file from the database and verify that the
        locations are correct.  Rollback the database transaction and once
        again reload the file.  Verify that the original locations are back.
        """
        testFileA = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10)
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testFileA.setLocation(["se1.fnal.gov"])

        myThread = threading.currentThread()
        myThread.transaction.begin()

        testFileA.setLocation(["se1.cern.ch"])
        testFileA.setLocation(["bunkse1.fnal.gov", "bunkse1.cern.ch"],
                              immediateSave=False)

        testFileB = DBSBufferFile(id=testFileA["id"])
        testFileB.load()

        goldenLocations = ["se1.fnal.gov", "se1.cern.ch"]

        for location in testFileB["locations"]:
            assert location in goldenLocations, \
                "ERROR: Unknown file location"
            goldenLocations.remove(location)

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing"

        myThread.transaction.rollback()
        testFileB.load()

        goldenLocations = ["se1.fnal.gov"]

        for location in testFileB["locations"]:
            assert location in goldenLocations, \
                "ERROR: Unknown file location"
            goldenLocations.remove(location)

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing"
        return

    def testLocationsConstructor(self):
        """
        _testLocationsConstructor_

        Test to make sure that locations passed into the File() constructor
        are loaded from and save to the database correctly.  Also test to make
        sure that the class behaves well when the location is passed in as a
        single string instead of a set.
        """
        testFileA = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10,
                                  locations=set(["se1.fnal.gov"]))
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        testFileB = DBSBufferFile(lfn="/this/is/a/lfn2", size=1024, events=10,
                                  locations="se1.fnal.gov")
        testFileB.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileB.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileB.addRun(Run(1, *[45]))
        testFileB.create()

        testFileC = DBSBufferFile(id=testFileA["id"])
        testFileC.load()

        goldenLocations = ["se1.fnal.gov"]
        for location in testFileC["locations"]:
            assert location in goldenLocations, \
                "ERROR: Unknown file location"
            goldenLocations.remove(location)

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing"

        testFileC = DBSBufferFile(id=testFileB["id"])
        testFileC.load()

        goldenLocations = ["se1.fnal.gov"]
        for location in testFileC["locations"]:
            assert location in goldenLocations, \
                "ERROR: Unknown file location"
            goldenLocations.remove(location)

        assert len(goldenLocations) == 0, \
            "ERROR: Some locations are missing"
        return

    def testAddRunSet(self):
        """
        _testAddRunSet_

        Test the ability to add run and lumi information to a file.
        """
        testFile = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10,
                                 locations="se1.fnal.gov")
        testFile.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                              appFam="RECO", psetHash="GIBBERISH",
                              configContent="MOREGIBBERISH")
        testFile.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")

        testFile.create()
        runSet = set()
        runSet.add(Run(1, *[45]))
        runSet.add(Run(2, *[67, 68]))
        testFile.addRunSet(runSet)

        assert (runSet - testFile["runs"]) == set(), \
            "Error: addRunSet is not updating set correctly"

    def testSetBlock(self):
        """
        _testSetBlock_

        Verify that the [Set|Get]Block DAOs work correctly.
        """
        myThread = threading.currentThread()

        dataset = "/Cosmics/CRUZET09-PromptReco-v1/RECO"

        uploadFactory = DAOFactory(package="WMComponent.DBS3Buffer",
                                   logger=myThread.logger,
                                   dbinterface=myThread.dbi)

        datasetAction = uploadFactory(classname="NewDataset")
        createAction = uploadFactory(classname="CreateBlocks")

        datasetAction.execute(datasetPath=dataset)

        newBlock = DBSBufferBlock(name="someblockname",
                                  location="se1.cern.ch",
                                  datasetpath=None)
        newBlock.setDataset(dataset, 'data', 'VALID')

        createAction.execute(blocks=[newBlock])

        setBlockAction = self.daoFactory(classname="DBSBufferFiles.SetBlock")
        getBlockAction = self.daoFactory(classname="DBSBufferFiles.GetBlock")

        testFile = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10,
                                 locations="se1.fnal.gov")
        testFile.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                              appFam="RECO", psetHash="GIBBERISH",
                              configContent="MOREGIBBERISH")
        testFile.setDatasetPath(dataset)

        testFile.create()

        setBlockAction.execute(lfn=testFile["lfn"], blockName="someblockname")
        blockName = getBlockAction.execute(lfn=testFile["lfn"])

        assert blockName[0][0] == "someblockname", \
            "Error: Incorrect block returned: %s" % blockName[0][0]
        return

    def testCountFilesDAO(self):
        """
        _testCountFilesDAO_

        Verify that the CountFiles DAO object functions correctly.
        """
        testFileA = DBSBufferFile(lfn="/this/is/a/lfnA", size=1024, events=10,
                                  locations="se1.fnal.gov")
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.create()

        testFileB = DBSBufferFile(lfn="/this/is/a/lfnB", size=1024, events=10,
                                  locations="se1.fnal.gov")
        testFileB.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileB.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileB.create()

        testFileC = DBSBufferFile(lfn="/this/is/a/lfnC", size=1024, events=10,
                                  locations="se1.fnal.gov")
        testFileC.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileC.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileC.create()

        countAction = self.daoFactory(classname="CountFiles")

        assert countAction.execute() == 3, \
            "Error: Wrong number of files counted in DBS Buffer."

        return

    def testAddParents(self):
        """
        _testAddParents_

        Verify that the addParents() method works correctly even if the parents
        do not already exist in the database.
        """
        myThread = threading.currentThread()

        testFile = DBSBufferFile(lfn="/this/is/a/lfnA", size=1024, events=10,
                                 locations="se1.fnal.gov")
        testFile.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                              appFam="RECO", psetHash="GIBBERISH",
                              configContent="MOREGIBBERISH")
        testFile.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFile.create()

        testParent = DBSBufferFile(lfn="/this/is/a/lfnB", size=1024, events=10,
                                   locations="se1.fnal.gov")
        testParent.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                appFam="RECO", psetHash="GIBBERISH",
                                configContent="MOREGIBBERISH")
        testParent.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RAW")
        testParent.create()

        goldenLFNs = ["lfn1", "lfn2", "lfn3", "/this/is/a/lfnB"]
        testFile.addParents(goldenLFNs)

        verifyFile = DBSBufferFile(id=testFile["id"])
        verifyFile.load(parentage=1)
        parentLFNs = verifyFile.getParentLFNs()

        for parentLFN in parentLFNs:
            self.assertTrue(parentLFN in goldenLFNs, "Error: unknown lfn %s" % parentLFN)
            goldenLFNs.remove(parentLFN)

        self.assertEqual(len(goldenLFNs), 0, "Error: missing LFNs...")

        # Check that the bogus dataset is listed as inDBS
        sqlCommand = """SELECT in_dbs FROM dbsbuffer_algo_dataset_assoc das
                          INNER JOIN dbsbuffer_dataset ds ON das.dataset_id = ds.id
                          WHERE ds.path = 'bogus'"""

        status = myThread.dbi.processData(sqlCommand)[0].fetchall()[0][0]

        self.assertEqual(status, 1)

        # Now make sure the dummy files are listed as being in DBS
        sqlCommand = """SELECT status FROM dbsbuffer_file df
                          INNER JOIN dbsbuffer_algo_dataset_assoc das ON das.id = df.dataset_algo
                          INNER JOIN dbsbuffer_dataset ds ON das.dataset_id = ds.id
                          WHERE ds.path = '/bogus/dataset/path' """

        status = myThread.dbi.processData(sqlCommand)[0].fetchall()

        for entry in status:
            self.assertEqual(entry, ('AlreadyInDBS',))

        return

    def testGetChildrenDAO(self):
        """
        _testGetChildrenDAO_

        Verify that the GetChildren DAO correctly returns the LFNs of a file's
        children.
        """
        testFileChildA = DBSBufferFile(lfn="/this/is/a/child/lfnA", size=1024,
                                       events=20)
        testFileChildA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                    appFam="RECO", psetHash="GIBBERISH",
                                    configContent="MOREGIBBERISH")
        testFileChildA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileChildB = DBSBufferFile(lfn="/this/is/a/child/lfnB", size=1024,
                                       events=20)
        testFileChildB.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                    appFam="RECO", psetHash="GIBBERISH",
                                    configContent="MOREGIBBERISH")
        testFileChildB.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileChildC = DBSBufferFile(lfn="/this/is/a/child/lfnC", size=1024,
                                       events=20)
        testFileChildC.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                    appFam="RECO", psetHash="GIBBERISH",
                                    configContent="MOREGIBBERISH")
        testFileChildC.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")

        testFileChildA.create()
        testFileChildB.create()
        testFileChildC.create()

        testFile = DBSBufferFile(lfn="/this/is/a/lfn", size=1024,
                                 events=10)
        testFile.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                              appFam="RECO", psetHash="GIBBERISH",
                              configContent="MOREGIBBERISH")
        testFile.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFile.create()

        testFileChildA.addParents([testFile["lfn"]])
        testFileChildB.addParents([testFile["lfn"]])
        testFileChildC.addParents([testFile["lfn"]])

        getChildrenAction = self.daoFactory(classname="DBSBufferFiles.GetChildren")
        childLFNs = getChildrenAction.execute(testFile["lfn"])

        assert len(childLFNs) == 3, \
            "ERROR: Parent does not have the right amount of children."

        goldenLFNs = ["/this/is/a/child/lfnA",
                      "/this/is/a/child/lfnB",
                      "/this/is/a/child/lfnC"]
        for childLFN in childLFNs:
            assert childLFN in goldenLFNs, \
                "ERROR: Unknown child lfn"
            goldenLFNs.remove(childLFN)

        return

    def testGetParentStatusDAO(self):
        """
        _testGetParentStatusDAO_

        Verify that the GetParentStatus DAO correctly returns the status of a
        file's children.
        """
        testFileChild = DBSBufferFile(lfn="/this/is/a/child/lfnA", size=1024,
                                      events=20)
        testFileChild.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                   appFam="RECO", psetHash="GIBBERISH",
                                   configContent="MOREGIBBERISH")
        testFileChild.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileChild.create()

        testFile = DBSBufferFile(lfn="/this/is/a/lfn", size=1024,
                                 events=10)
        testFile.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                              appFam="RECO", psetHash="GIBBERISH",
                              configContent="MOREGIBBERISH")
        testFile.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFile.create()

        testFileChild.addParents([testFile["lfn"]])

        getStatusAction = self.daoFactory(classname="DBSBufferFiles.GetParentStatus")
        parentStatus = getStatusAction.execute(testFileChild["lfn"])

        assert len(parentStatus) == 1, \
            "ERROR: Wrong number of statuses returned."
        assert parentStatus[0] == "NOTUPLOADED", \
            "ERROR: Wrong status returned."

        return

    def testSetLocationByLFN(self):
        """
        _testSetLocationByLFN_

        """
        testFileA = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10)
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.addRun(Run(1, *[45]))
        testFileA.create()

        setLocationAction = self.daoFactory(classname="DBSBufferFiles.SetLocationByLFN")
        setLocationAction.execute(binds=[{'lfn': "/this/is/a/lfn", 'pnn': 'se1.cern.ch'}])

        testFileB = DBSBufferFile(id=testFileA["id"])
        testFileB.load()

        self.assertEqual(testFileB['locations'], set(['se1.cern.ch']))

        return

    def testAddCKSumByLFN(self):
        """
        _testAddCKSumByLFN_

        """

        testFileA = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10)
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.create()

        setCksumAction = self.daoFactory(classname="DBSBufferFiles.AddChecksumByLFN")
        binds = [{'lfn': "/this/is/a/lfn", 'cktype': 'adler32', 'cksum': 201},
                 {'lfn': "/this/is/a/lfn", 'cktype': 'cksum', 'cksum': 101}]
        setCksumAction.execute(bulkList=binds)

        testFileB = DBSBufferFile(id=testFileA["id"])
        testFileB.load()

        self.assertEqual(testFileB['checksums'], {'adler32': '201', 'cksum': '101'})

        return

    def testBulkLoad(self):
        """
        _testBulkLoad_

        Can we load in bulk?
        """

        addToBuffer = DBSBufferUtil()

        testFileChildA = DBSBufferFile(lfn="/this/is/a/child/lfnA", size=1024,
                                       events=20)
        testFileChildA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                    appFam="RECO", psetHash="GIBBERISH",
                                    configContent="MOREGIBBERISH")
        testFileChildA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileChildB = DBSBufferFile(lfn="/this/is/a/child/lfnB", size=1024,
                                       events=20)
        testFileChildB.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                    appFam="RECO", psetHash="GIBBERISH",
                                    configContent="MOREGIBBERISH")
        testFileChildB.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileChildC = DBSBufferFile(lfn="/this/is/a/child/lfnC", size=1024,
                                       events=20)
        testFileChildC.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                                    appFam="RECO", psetHash="GIBBERISH",
                                    configContent="MOREGIBBERISH")
        testFileChildC.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")

        testFileChildA.create()
        testFileChildB.create()
        testFileChildC.create()

        testFileChildA.setLocation(["se1.fnal.gov", "se1.cern.ch"])
        testFileChildB.setLocation(["se1.fnal.gov", "se1.cern.ch"])
        testFileChildC.setLocation(["se1.fnal.gov", "se1.cern.ch"])

        runSet = set()
        runSet.add(Run(1, *[45]))
        runSet.add(Run(2, *[67, 68]))
        testFileChildA.addRunSet(runSet)
        testFileChildB.addRunSet(runSet)
        testFileChildC.addRunSet(runSet)

        testFileChildA.save()
        testFileChildB.save()
        testFileChildC.save()

        setCksumAction = self.daoFactory(classname="DBSBufferFiles.AddChecksumByLFN")
        binds = [{'lfn': "/this/is/a/child/lfnA", 'cktype': 'adler32', 'cksum': 201},
                 {'lfn': "/this/is/a/child/lfnA", 'cktype': 'cksum', 'cksum': 101},
                 {'lfn': "/this/is/a/child/lfnB", 'cktype': 'adler32', 'cksum': 201},
                 {'lfn': "/this/is/a/child/lfnB", 'cktype': 'cksum', 'cksum': 101},
                 {'lfn': "/this/is/a/child/lfnC", 'cktype': 'adler32', 'cksum': 201},
                 {'lfn': "/this/is/a/child/lfnC", 'cktype': 'cksum', 'cksum': 101}]
        setCksumAction.execute(bulkList=binds)

        testFile = DBSBufferFile(lfn="/this/is/a/lfn", size=1024,
                                 events=10)
        testFile.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                              appFam="RECO", psetHash="GIBBERISH",
                              configContent="MOREGIBBERISH")
        testFile.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFile.create()

        testFileChildA.addParents([testFile["lfn"]])
        testFileChildB.addParents([testFile["lfn"]])
        testFileChildC.addParents([testFile["lfn"]])

        binds = [{'id': testFileChildA.exists()},
                 {'id': testFileChildB.exists()},
                 {'id': testFileChildC.exists()}]

        listOfFiles = addToBuffer.loadDBSBufferFilesBulk(fileObjs=binds)

        # print listOfFiles


        compareList = ['locations', 'psetHash', 'configContent', 'appName',
                       'appVer', 'appFam', 'events', 'datasetPath', 'runs']

        for f in listOfFiles:
            self.assertTrue(f['lfn'] in ["/this/is/a/child/lfnA", "/this/is/a/child/lfnB",
                                         "/this/is/a/child/lfnC"],
                            "Unknown file in loaded results")
            self.assertEqual(f['checksums'], {'adler32': '201', 'cksum': '101'})
            for parent in f['parents']:
                self.assertEqual(parent['lfn'], testFile['lfn'])
            for key in compareList:
                self.assertEqual(f[key], testFileChildA[key])

    def testProperties(self):
        """
        _testProperties_

        Test added tags that use DBSBuffer to transfer from workload to DBS
        """

        testFileA = DBSBufferFile(lfn="/this/is/a/lfn", size=1024, events=10)
        testFileA.setAlgorithm(appName="cmsRun", appVer="CMSSW_2_1_8",
                               appFam="RECO", psetHash="GIBBERISH",
                               configContent="MOREGIBBERISH")
        testFileA.setDatasetPath("/Cosmics/CRUZET09-PromptReco-v1/RECO")
        testFileA.setValidStatus(validStatus="VALID")
        testFileA.setProcessingVer(ver="123")
        testFileA.setAcquisitionEra(era="AcqEra")
        testFileA.setGlobalTag(globalTag="GlobalTag")
        testFileA.setDatasetParent(datasetParent="Parent")
        testFileA.create()

        return


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_LogCollect_

Basic Emulator for LogCollect Step

"""
from __future__ import print_function

import os
import os.path
import re
import shutil

from WMCore.WMSpec.Steps.Emulator import Emulator

class LogCollect(Emulator):
    """
    _LogCollect_

    Emulate the execution of a LogCollect Step

    """
    def pre(self):
        """
        _pre_

        Emulate the stage out pre step

        """
        return None

    def execute(self):
        """
        _execute_

        Emulate LogCollect execution

        """
        self.step.section_("execution")
        self.step.execution.exitStatus = 0
        self.step.section_("emulation")
        self.step.emulation.emulatedBy = str(self.__class__.__name__)

        print("Emulating LogCollect Step")

        output = getattr(self.report.data, self.stepName)
        output.outputPFN = 'ThisIsAPFN'
        output.PNN       = 'ThisIsAPNN'
        output.LFN       = 'ThisIsALFN'

        print("Have done emulation")
        print(self.report.data)

        return



    def post(self):
        """
        _post_

        Emulate post stage out

        """
        return None



"""
_CountFinishedSubscriptionsByWorkflow_

Oracle implementation of CountFinishedSubscriptionsByWorkflow
"""
from __future__ import division, print_function

from WMCore.WMBS.MySQL.Subscriptions.CountFinishedSubscriptionsByWorkflow \
    import CountFinishedSubscriptionsByWorkflow as MySQLCountFinishedSubscriptionsByWorkflow


class CountFinishedSubscriptionsByWorkflow(MySQLCountFinishedSubscriptionsByWorkflow):
    pass



#!/usr/bin/env python
"""
_ListByState_

Oracle implementation of ListByState

Created on July 10, 2013

@author: tsarangi
"""

from WMCore.WMBS.MySQL.Jobs.ListByState import ListByState as MySQLListByState

class ListByState(MySQLListByState):
    pass
